"""
 * Copyright 2022 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
"""

import mock
import pyspark

from dataproc_templates.hbase.hbase_to_gcs import HbaseToGCSTemplate
import dataproc_templates.util.template_constants as constants


class TestHbaseToGCSTemplate:
    """
    Test suite for HbaseToGCSTemplate
    """

    def test_parse_args(self):
        """Tests HbaseToGCSTemplate.parse_args()"""

        hive_to_gcs_template = HbaseToGCSTemplate()
        parsed_args = hive_to_gcs_template.parse_args(
            ["--hbase.gcs.output.location=gs://test",
             "--hbase.gcs.output.format=parquet",
             "--hbase.gcs.output.mode=overwrite",
             "--hbase.gcs.catalog.json={key:value}"])

        assert parsed_args["hbase.gcs.output.location"] == "gs://test"
        assert parsed_args["hbase.gcs.output.format"] == "parquet"
        assert parsed_args["hbase.gcs.output.mode"] == "overwrite"
        assert parsed_args["hbase.gcs.catalog.json"] == '{key:value}'

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run(self, mock_spark_session):
        """Tests HbaseToGCSTemplate runs for parquet format output"""

        hbase_to_gcs_template = HbaseToGCSTemplate()
        mock_parsed_args = hbase_to_gcs_template.parse_args(
            ["--hbase.gcs.output.location=gs://test",
             "--hbase.gcs.output.format=parquet",
             "--hbase.gcs.output.mode=overwrite",
             "--hbase.gcs.catalog.json={key:value}"])
        mock_spark_session.read.format() \
                .options().option() \
                .load.return_value = mock_spark_session.dataframe.DataFrame
        hbase_to_gcs_template.run(mock_spark_session, mock_parsed_args)

        mock_spark_session.read.format \
                .assert_called_with(constants.FORMAT_HBASE)
        mock_spark_session.read.format() \
                .options.assert_called_with(catalog='{key:value}')
        mock_spark_session.read.format() \
                .options().option.assert_called_with('hbase.spark.use.hbasecontext', "false")
        mock_spark_session.read.format() \
                .options().option().load.assert_called_once_with()

        mock_spark_session.dataframe.DataFrame.write \
            .mode.assert_called_once_with(constants.OUTPUT_MODE_OVERWRITE)
        mock_spark_session.dataframe.DataFrame.write \
            .mode() \
            .parquet.assert_called_once_with("gs://test")



from typing import Dict, Sequence, Optional, Any
from logging import Logger
import argparse
import pprint

from pyspark.sql import SparkSession

from dataproc_templates import BaseTemplate
import dataproc_templates.util.template_constants as constants

__all__ = ['PubSubLiteToGCSTemplate']

class PubSubLiteToGCSTemplate(BaseTemplate):
    """
    Dataproc template implementing exports from PubSubLite to GCS
    """

    @staticmethod
    def parse_args(args: Optional[Sequence[str]] = None) -> Dict[str, Any]:
        parser: argparse.ArgumentParser = argparse.ArgumentParser()

        parser.add_argument(
            f'--{constants.PUBSUBLITE_TO_GCS_INPUT_SUBSCRIPTION_URL}',
            dest=constants.PUBSUBLITE_TO_GCS_INPUT_SUBSCRIPTION_URL,
            required=True,
            help='PubSubLite to GCS Input subscription url'
        )
        parser.add_argument(
            f'--{constants.PUBSUBLITE_TO_GCS_WRITE_MODE}',
            dest=constants.PUBSUBLITE_TO_GCS_WRITE_MODE,
            required=False,
            default=constants.OUTPUT_MODE_APPEND,
            help=(
                'Output write mode '
                '(one of: append,overwrite,ignore,errorifexists) '
                '(Defaults to append)'
            ),
            choices=[
                constants.OUTPUT_MODE_OVERWRITE,
                constants.OUTPUT_MODE_APPEND,
                constants.OUTPUT_MODE_IGNORE,
                constants.OUTPUT_MODE_ERRORIFEXISTS
            ]
        )
        parser.add_argument(
            f'--{constants.PUBSUBLITE_TO_GCS_OUTPUT_LOCATION}',
            dest=constants.PUBSUBLITE_TO_GCS_OUTPUT_LOCATION,
            required=True,
            help='GCS output Bucket URL'
        )
        parser.add_argument(
            f'--{constants.PUBSUBLITE_CHECKPOINT_LOCATION}',
            dest=constants.PUBSUBLITE_CHECKPOINT_LOCATION,
            required=True,
            help='Temporary folder for checkpoint location'
        )
        parser.add_argument(
            f'--{constants.PUBSUBLITE_TO_GCS_OUTPUT_FORMAT}',
            dest=constants.PUBSUBLITE_TO_GCS_OUTPUT_FORMAT,
            required=False,
            default=constants.FORMAT_CSV,
            help=(
                'Output Format to GCS '
                '(one of: json, csv, avro, parquet) '
                '(Defaults to csv)'
            ),
            choices=[
                constants.FORMAT_AVRO,
                constants.FORMAT_CSV,
                constants.FORMAT_JSON,
                constants.FORMAT_PRQT
            ]
        )
        parser.add_argument(
            f'--{constants.PUBSUBLITE_TO_GCS_TIMEOUT}',
            dest=constants.PUBSUBLITE_TO_GCS_TIMEOUT,
            required=True,
            type=int,
            help=('Time for which subscriptions will be read')
        )
        parser.add_argument(
            f'--{constants.PUBSUBLITE_TO_GCS_PROCESSING_TIME}',
            dest=constants.PUBSUBLITE_TO_GCS_PROCESSING_TIME,
            required=True,
            help=('Time interval at which the query will be triggered to process input data')
        )

        known_args: argparse.Namespace
        known_args, _ = parser.parse_known_args(args)

        return vars(known_args)

    def run(self, spark: SparkSession, args: Dict[str, Any]) -> None:

        logger: Logger = self.get_logger(spark=spark)

        # Arguments
        input_subscription_url: str = args[constants.PUBSUBLITE_TO_GCS_INPUT_SUBSCRIPTION_URL]
        output_location: str = args[constants.PUBSUBLITE_TO_GCS_OUTPUT_LOCATION]
        output_mode: str = args[constants.PUBSUBLITE_TO_GCS_WRITE_MODE]
        pubsublite_checkpoint_location: str = args[constants.PUBSUBLITE_CHECKPOINT_LOCATION]
        output_format: str = args[constants.PUBSUBLITE_TO_GCS_OUTPUT_FORMAT]
        timeout: int = args[constants.PUBSUBLITE_TO_GCS_TIMEOUT]
        processing_time: str = args[constants.PUBSUBLITE_TO_GCS_PROCESSING_TIME]

        logger.info(
            "Starting PubSubLite to GCS spark job with parameters:\n"
            f"{pprint.pformat(args)}"
        )

        # Read
        input_data=(spark.readStream \
            .format(constants.FORMAT_PUBSUBLITE) \
            .option(f"{constants.FORMAT_PUBSUBLITE}.subscription", input_subscription_url) \
            .load())

        # Write
        query = (input_data.writeStream \
            .format(output_format) \
            .option("checkpointLocation", pubsublite_checkpoint_location) \
            .option("path", output_location) \
            .outputMode(output_mode) \
            .trigger(processingTime=processing_time) \
            .start())

        # Wait for some time (must be >= 60 seconds) to start receiving messages.
        query.awaitTermination(timeout)
        query.stop()


"""
 * Copyright 2022 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
"""

import mock
import pyspark

from dataproc_templates.gcs.text_to_bigquery import TextToBigQueryTemplate
import dataproc_templates.util.template_constants as constants


class TestTextToBigQueryTemplate:
    """
    Test suite for TextToBigQueryTemplate
    """

    def test_parse_args(self):
        """Tests TextToBigQueryTemplate.parse_args()"""

        text_to_bigquery_template = TextToBigQueryTemplate()
        parsed_args = text_to_bigquery_template.parse_args(
            ["--text.bigquery.input.compression=bzip2",
             "--text.bigquery.output.mode=append",
             "--text.bigquery.input.location=gs://test",
             "--text.bigquery.output.dataset=dataset",
             "--text.bigquery.output.table=table",
             "--text.bigquery.temp.bucket.name=bucket",
             "--text.bigquery.input.sep=|"])

        assert parsed_args["text.bigquery.input.compression"] == "bzip2"
        assert parsed_args["text.bigquery.output.mode"] == "append"
        assert parsed_args["text.bigquery.input.location"] == "gs://test"
        assert parsed_args["text.bigquery.output.dataset"] == "dataset"
        assert parsed_args["text.bigquery.output.table"] == "table"
        assert parsed_args["text.bigquery.temp.bucket.name"] == "bucket"
        assert parsed_args["text.bigquery.input.sep"] == "|"

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_bzip2(self, mock_spark_session):
        """Tests TextToBigqueryTemplate runs with | delimiter and bzip2 compression format"""

        text_to_bigquery_template = TextToBigQueryTemplate()
        mock_parsed_args = text_to_bigquery_template.parse_args(
            ["--text.bigquery.input.location=gs://test",
             "--text.bigquery.input.header=false",
             "--text.bigquery.output.dataset=dataset",
             "--text.bigquery.output.table=table",
             "--text.bigquery.input.compression=bzip2",
             "--text.bigquery.temp.bucket.name=bucket",
             "--text.bigquery.output.mode=ignore",
             "--text.bigquery.input.sep=|"])

        mock_spark_session.read.format().options().load.return_value = mock_spark_session.dataframe.DataFrame
        text_to_bigquery_template.run(mock_spark_session, mock_parsed_args)

        mock_spark_session.read.format.assert_called_with(
            constants.FORMAT_CSV)
        mock_spark_session.read.format().options.assert_called_with(**{
            constants.CSV_HEADER: 'false',
            constants.CSV_INFER_SCHEMA: 'true',
            constants.CSV_SEP: "|",
        })
        mock_spark_session.read.format().options().load.assert_called_once_with("gs://test")

        mock_spark_session.dataframe.DataFrame.write.format.assert_called_once_with(
            constants.FORMAT_BIGQUERY)
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option.assert_called_once_with(constants.TABLE, "dataset.table")
        mock_spark_session.dataframe.DataFrame.write.format().option(
        ).option.assert_called_once_with(constants.GCS_BQ_TEMP_BUCKET, "bucket")
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option().mode.assert_called_once_with(constants.OUTPUT_MODE_IGNORE)
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option().mode().save.assert_called_once()

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_lz4(self, mock_spark_session):
        """Tests TextToBigqueryTemplate runs with | delimiter and lz4 compression format"""

        text_to_bigquery_template = TextToBigQueryTemplate()
        mock_parsed_args = text_to_bigquery_template.parse_args(
            ["--text.bigquery.input.location=gs://test",
             "--text.bigquery.input.header=false",
             "--text.bigquery.input.compression=lz4",
             "--text.bigquery.input.comment=#",
             "--text.bigquery.input.timestampntzformat=yyyy-MM-dd'T'HH:mm:ss",
             "--text.bigquery.output.dataset=dataset",
             "--text.bigquery.output.table=table",
             "--text.bigquery.temp.bucket.name=bucket",
             "--text.bigquery.output.mode=ignore",
             "--text.bigquery.input.sep=|"])

        mock_spark_session.read.format().options().load.return_value = mock_spark_session.dataframe.DataFrame
        text_to_bigquery_template.run(mock_spark_session, mock_parsed_args)

        mock_spark_session.read.format.assert_called_with(
            constants.FORMAT_CSV)
        mock_spark_session.read.format().options.assert_called_with(**{
            constants.CSV_HEADER: 'false',
            constants.CSV_INFER_SCHEMA: 'true',
            constants.CSV_SEP: "|",
            constants.CSV_COMMENT: "#",
            constants.CSV_TIMESTAMPNTZFORMAT: "yyyy-MM-dd'T'HH:mm:ss",
        })
        mock_spark_session.read.format().options().load.assert_called_once_with("gs://test")

        mock_spark_session.dataframe.DataFrame.write.format.assert_called_once_with(
            constants.FORMAT_BIGQUERY)
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option.assert_called_once_with(constants.TABLE, "dataset.table")
        mock_spark_session.dataframe.DataFrame.write.format().option(
        ).option.assert_called_once_with(constants.GCS_BQ_TEMP_BUCKET, "bucket")
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option().mode.assert_called_once_with(constants.OUTPUT_MODE_IGNORE)
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option().mode().save.assert_called_once()

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_deflate(self, mock_spark_session):
        """Tests TextToBigqueryTemplate runs with | delimiter and deflate compression format"""

        text_to_bigquery_template = TextToBigQueryTemplate()
        mock_parsed_args = text_to_bigquery_template.parse_args(
            ["--text.bigquery.input.location=gs://test",
             "--text.bigquery.input.header=false",
             "--text.bigquery.input.compression=deflate",
             "--text.bigquery.input.mode=FAILFAST",
             "--text.bigquery.output.dataset=dataset",
             "--text.bigquery.output.table=table",
             "--text.bigquery.temp.bucket.name=bucket",
             "--text.bigquery.output.mode=ignore",
             "--text.bigquery.input.sep=|"])

        mock_spark_session.read.format().options().load.return_value = mock_spark_session.dataframe.DataFrame
        text_to_bigquery_template.run(mock_spark_session, mock_parsed_args)

        mock_spark_session.read.format.assert_called_with(
            constants.FORMAT_CSV)
        mock_spark_session.read.format().options.assert_called_with(**{
            constants.CSV_HEADER: 'false',
            constants.CSV_INFER_SCHEMA: 'true',
            constants.CSV_MODE: 'FAILFAST',
            constants.CSV_SEP: "|",
        })
        mock_spark_session.read.format().options().load.assert_called_once_with("gs://test")

        mock_spark_session.dataframe.DataFrame.write.format.assert_called_once_with(
            constants.FORMAT_BIGQUERY)
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option.assert_called_once_with(constants.TABLE, "dataset.table")
        mock_spark_session.dataframe.DataFrame.write.format().option(
        ).option.assert_called_once_with(constants.GCS_BQ_TEMP_BUCKET, "bucket")
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option().mode.assert_called_once_with(constants.OUTPUT_MODE_IGNORE)
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option().mode().save.assert_called_once()

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_gzip(self, mock_spark_session):
        """Tests TextToBigqueryTemplate runs with | delimiter and gzip compression format"""

        text_to_bigquery_template = TextToBigQueryTemplate()
        mock_parsed_args = text_to_bigquery_template.parse_args(
            ["--text.bigquery.input.location=gs://test",
             "--text.bigquery.input.maxcolumns=100",
             "--text.bigquery.input.maxcharspercolumn=64",
             "--text.bigquery.input.compression=gzip",
             "--text.bigquery.output.dataset=dataset",
             "--text.bigquery.output.table=table",
             "--text.bigquery.temp.bucket.name=bucket",
             "--text.bigquery.output.mode=ignore",
             "--text.bigquery.input.sep=|"])
        mock_spark_session.read.format().options(
        ).load.return_value = mock_spark_session.dataframe.DataFrame
        text_to_bigquery_template.run(mock_spark_session, mock_parsed_args)

        mock_spark_session.read.format.assert_called_with(
            constants.FORMAT_CSV)
        mock_spark_session.read.format().options.assert_called_with(**{
            constants.CSV_HEADER: 'true',
            constants.CSV_INFER_SCHEMA: 'true',
            constants.CSV_MAXCOLUMNS: '100',
            constants.CSV_MAXCHARSPERCOLUMN: '64',
            constants.CSV_SEP: "|",
        })
        mock_spark_session.read.format().options().load.assert_called_once_with("gs://test")

        mock_spark_session.dataframe.DataFrame.write.format.assert_called_once_with(
            constants.FORMAT_BIGQUERY)
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option.assert_called_once_with(constants.TABLE, "dataset.table")
        mock_spark_session.dataframe.DataFrame.write.format().option(
        ).option.assert_called_once_with(constants.GCS_BQ_TEMP_BUCKET, "bucket")
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option().mode.assert_called_once_with(constants.OUTPUT_MODE_IGNORE)
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option().mode().save.assert_called_once()



#
# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import argparse
import pprint
from logging import Logger
from typing import Any, Dict, Optional, Sequence
import dataproc_templates.util.template_constants as constants
from dataproc_templates import BaseTemplate
from pyspark.sql import SparkSession
from datetime import datetime

class HiveDDLExtractorTemplate(BaseTemplate):
    """
    Dataproc template implementing exports from Hive to BigQuery
    """

    @staticmethod
    def parse_args(args: Optional[Sequence[str]] = None) -> Dict[str, Any]:
        parser: argparse.ArgumentParser = argparse.ArgumentParser()

        parser.add_argument(
            f'--{constants.HIVE_DDL_EXTRACTOR_INPUT_DATABASE}',
            dest=constants.HIVE_DDL_EXTRACTOR_INPUT_DATABASE,
            required=True,
            help='Hive database for importing data to BigQuery'
        )

        parser.add_argument(
            f'--{constants.HIVE_DDL_EXTRACTOR_OUTPUT_GCS_PATH}',
            dest=constants.HIVE_DDL_EXTRACTOR_OUTPUT_GCS_PATH,
            required=True,
            help='GCS output path'
        )

        parser.add_argument(
            f'--{constants.HIVE_DDL_CONSIDER_SPARK_TABLES}',
            dest=constants.HIVE_DDL_CONSIDER_SPARK_TABLES,
            required=False,
            default=False,
            help='Flag to extract DDL of Spark tables'
        )

        parser.add_argument(
            f'--{constants.HIVE_DDL_TRANSLATION_DISPOSITION}',
            dest=constants.HIVE_DDL_TRANSLATION_DISPOSITION,
            required=False,
            default=False,
            help='Remove location parameter from HIVE DDL if set to TRUE, to be compatible with BigQuery SQL translator'
        )

        known_args: argparse.Namespace
        known_args, _ = parser.parse_known_args(args)

        return vars(known_args)


    def run(self, spark: SparkSession, args: Dict[str, Any]) -> None:
        """
        
        Dataproc template allowing the extraction of DDLs from Hive Metastore

        """
        logger: Logger = self.get_logger(spark=spark)

        hive_database: str = args[constants.HIVE_DDL_EXTRACTOR_INPUT_DATABASE]
        gcs_output_path: str = args[constants.HIVE_DDL_EXTRACTOR_OUTPUT_GCS_PATH]
        spark_tbls_flag: bool = args[constants.HIVE_DDL_CONSIDER_SPARK_TABLES]
        remove_location_flag: bool = args[constants.HIVE_DDL_TRANSLATION_DISPOSITION]

        logger.info(
            "Starting Hive DDL Extraction job with parameters:\n"
            f"{pprint.pformat(args)}"
        )

        def get_ddl(hive_database, table_name):
            spark_tbls_opt = "" if str(spark_tbls_flag).upper() == "TRUE" else "AS SERDE"
            ddl_str = spark.sql(f"SHOW CREATE TABLE {hive_database}.{table_name} {spark_tbls_opt}").rdd.map(lambda x: x[0]).collect()[0]
            ddl_str = ddl_str + ";" if str(remove_location_flag).upper() != "TRUE" else ddl_str.split("\nLOCATION '")[0].split("\nUSING ")[0]+";"
            return ddl_str

        ct = datetime.now().strftime("%m-%d-%Y %H.%M.%S")
        output_path = gcs_output_path+"/"+hive_database+"/"+str(ct)

        tables_names = spark.sql(f"SHOW TABLES IN {hive_database}").select("tableName")
        tables_name_list = tables_names.rdd.map(lambda x: x[0]).collect()
        tables_ddls = [get_ddl(hive_database, table_name) for table_name in tables_name_list]

        ddls_rdd = spark.sparkContext.parallelize(tables_ddls)
        ddls_rdd.coalesce(1).saveAsTextFile(output_path)



"""
 * Copyright 2022 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
"""

import argparse
from typing import List

import pytest

import dataproc_templates.util.template_constants as constants
from dataproc_templates.util.template_constants import \
    get_csv_input_spark_options as in_csv_opt
from dataproc_templates.util.template_constants import \
    get_csv_output_spark_options as out_csv_opt

from dataproc_templates import TemplateName
from dataproc_templates.util.argument_parsing import (
    add_spark_options,
    get_template_name,
    get_log_level,
)


def test_get_valid_template_names():
    """Tests valid template names"""
    template_names: List[str] = ["GCSTOBIGQUERY", "BIGQUERYTOGCS",
                                 "TEXTTOBIGQUERY"]

    for template_name in template_names:
        parsed_template_name: TemplateName = get_template_name(
            args=["--template", template_name]
        )
        assert template_name == parsed_template_name.value


def test_get_invalid_template_name():
    """Tests that an invalid template name raises an error"""
    template_name = "GCSTOSMALLQUERY"
    with pytest.raises(SystemExit):
        get_template_name(["--template=" + template_name])


def test_get_valid_log_level():
    """Tests valid log levels"""
    log_levels: List[str] = [
        "ALL",
        "DEBUG",
        "ERROR",
        "FATAL",
        "INFO",
        "OFF",
        "TRACE",
        "WARN",
    ]

    for log_level in log_levels:
        parsed_log_level: str = get_log_level(args=["--log_level", log_level])
        assert log_level == parsed_log_level


def test_get_invalid_log_level():
    """Tests that an invalid log level raises an error"""
    log_level = "AWESOME_LOG_LEVEL"
    with pytest.raises(SystemExit):
        get_template_name(["--log_level=" + log_level])


def test_add_spark_options():
    for option_set, option_prefix in [
        # Read options
        (in_csv_opt("mock.in.prefix."), "mock.in.prefix"),
        # Write options
        (out_csv_opt("mock.out.prefix."), "mock.out.prefix"),
    ]:
        parser: argparse.ArgumentParser = argparse.ArgumentParser()
        add_spark_options(parser, option_set)
        known_args, _ = parser.parse_known_args()
        for k in option_set:
            assert k.startswith(
                option_prefix
            ), f"Attribute {k} does not start with {option_prefix}"
            assert hasattr(
                known_args, k
            ), f"Attribute {k} missing from {option_prefix} args"
            assert (
                option_set[k] in constants.SPARK_OPTIONS
            ), f"Attribute {k} maps to invalid Spark option {option_set[k]}"
            spark_option = option_set[k]
            assert (
                constants.SPARK_OPTIONS[spark_option].get(
                    constants.OPTION_HELP)
                or constants.SPARK_OPTIONS[spark_option].get(
                    constants.OPTION_READ_HELP)
                or constants.SPARK_OPTIONS[spark_option].get(
                    constants.OPTION_WRITE_HELP
                )
            ), f"Attribute {spark_option} has no help text in SPARK_OPTIONS"



# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Dict, Sequence, Optional, Any
from logging import Logger
import argparse
import pprint
import sys

from pyspark.sql import SparkSession, DataFrame, DataFrameWriter

from dataproc_templates import BaseTemplate
from dataproc_templates.util.argument_parsing import add_spark_options
from dataproc_templates.util.dataframe_writer_wrappers import persist_dataframe_to_cloud_storage
import dataproc_templates.util.template_constants as constants


__all__ = ['SnowflakeToGCSTemplate']


class SnowflakeToGCSTemplate(BaseTemplate):
    """
    Dataproc template implementing loads from Snowflake to Cloud Storage
    """

    @staticmethod
    def parse_args(args: Optional[Sequence[str]] = None) -> Dict[str, Any]:
        parser: argparse.ArgumentParser = argparse.ArgumentParser()

        parser.add_argument(
            f'--{constants.SNOWFLAKE_TO_GCS_SF_URL}',
            dest=constants.SNOWFLAKE_TO_GCS_SF_URL,
            required=True,
            help='Snowflake connection URL'
        )

        parser.add_argument(
            f'--{constants.SNOWFLAKE_TO_GCS_SF_USER}',
            dest=constants.SNOWFLAKE_TO_GCS_SF_USER,
            required=True,
            help='Snowflake user name'
        )

        parser.add_argument(
            f'--{constants.SNOWFLAKE_TO_GCS_SF_PASSWORD}',
            dest=constants.SNOWFLAKE_TO_GCS_SF_PASSWORD,
            required=True,
            help='Snowflake password'
        )

        parser.add_argument(
            f'--{constants.SNOWFLAKE_TO_GCS_SF_DATABASE}',
            dest=constants.SNOWFLAKE_TO_GCS_SF_DATABASE,
            required=False,
            default="",
            help='Snowflake database name'
        )

        parser.add_argument(
            f'--{constants.SNOWFLAKE_TO_GCS_SF_WAREHOUSE}',
            dest=constants.SNOWFLAKE_TO_GCS_SF_WAREHOUSE,
            required=False,
            default="",
            help='Snowflake datawarehouse name'
        )

        parser.add_argument(
            f'--{constants.SNOWFLAKE_TO_GCS_SF_AUTOPUSHDOWN}',
            dest=constants.SNOWFLAKE_TO_GCS_SF_AUTOPUSHDOWN,
            required=False,
            default="on",
            help='Snowflake Autopushdown (on|off)'
        )

        parser.add_argument(
            f'--{constants.SNOWFLAKE_TO_GCS_SF_SCHEMA}',
            dest=constants.SNOWFLAKE_TO_GCS_SF_SCHEMA,
            required=False,
            default="",
            help='Snowflake Schema, the source table belongs to'
        )

        parser.add_argument(
            f'--{constants.SNOWFLAKE_TO_GCS_SF_TABLE}',
            dest=constants.SNOWFLAKE_TO_GCS_SF_TABLE,
            required=False,
            default="",
            help='Snowflake table name'
        )

        parser.add_argument(
            f'--{constants.SNOWFLAKE_TO_GCS_SF_QUERY}',
            dest=constants.SNOWFLAKE_TO_GCS_SF_QUERY,
            required=False,
            default="",
            help='Query to be executed on Snowflake to fetch \
                the desired dataset for migration'
        )

        parser.add_argument(
            f'--{constants.SNOWFLAKE_TO_GCS_OUTPUT_LOCATION}',
            dest=constants.SNOWFLAKE_TO_GCS_OUTPUT_LOCATION,
            required=True,
            help='Cloud Storage output location where the migrated data will be placed'
        )

        parser.add_argument(
            f'--{constants.SNOWFLAKE_TO_GCS_OUTPUT_MODE}',
            dest=constants.SNOWFLAKE_TO_GCS_OUTPUT_MODE,
            required=False,
            default=constants.OUTPUT_MODE_APPEND,
            help=(
                'Output write mode '
                '(one of: append,overwrite,ignore,errorifexists) '
                '(Defaults to append)'
            ),
            choices=[
                constants.OUTPUT_MODE_OVERWRITE,
                constants.OUTPUT_MODE_APPEND,
                constants.OUTPUT_MODE_IGNORE,
                constants.OUTPUT_MODE_ERRORIFEXISTS
            ]
        )

        parser.add_argument(
            f'--{constants.SNOWFLAKE_TO_GCS_OUTPUT_FORMAT}',
            dest=constants.SNOWFLAKE_TO_GCS_OUTPUT_FORMAT,
            required=False,
            default=constants.FORMAT_CSV,
            help=(
                'Output write format '
                '(one of: avro,parquet,csv,json)'
                '(Defaults to csv)'
            ),
            choices=[
                constants.FORMAT_AVRO,
                constants.FORMAT_PRQT,
                constants.FORMAT_CSV,
                constants.FORMAT_JSON
            ]
        )

        parser.add_argument(
            f'--{constants.SNOWFLAKE_TO_GCS_PARTITION_COLUMN}',
            dest=constants.SNOWFLAKE_TO_GCS_PARTITION_COLUMN,
            required=False,
            default="",
            help='Column name to partition data by, in Cloud Storage bucket'
        )
        add_spark_options(parser, constants.get_csv_output_spark_options("snowflake.gcs.output."))

        known_args: argparse.Namespace
        known_args, _ = parser.parse_known_args(args)

        if ((not getattr(known_args, constants.SNOWFLAKE_TO_GCS_SF_DATABASE)
                or not getattr(known_args, constants.SNOWFLAKE_TO_GCS_SF_SCHEMA)
                or not getattr(known_args, constants.SNOWFLAKE_TO_GCS_SF_TABLE))
            and not getattr(known_args, constants.SNOWFLAKE_TO_GCS_SF_QUERY)):

            sys.exit("ArgumentParser Error: Either of snowflake.to.gcs.sf.database, snowflake.to.gcs.sf.schema and snowflake.to.gcs.sf.table "
                        + "OR snowflake.to.gcs.sf.query needs to be provided as argument to read data from Snowflake")

        elif ((getattr(known_args, constants.SNOWFLAKE_TO_GCS_SF_DATABASE)
                or getattr(known_args, constants.SNOWFLAKE_TO_GCS_SF_SCHEMA)
                or getattr(known_args, constants.SNOWFLAKE_TO_GCS_SF_TABLE))
            and getattr(known_args, constants.SNOWFLAKE_TO_GCS_SF_QUERY)):

            sys.exit("ArgumentParser Error: All three snowflake.to.gcs.sf.database, snowflake.to.gcs.sf.schema and snowflake.to.gcs.sf.table "
                        + "AND snowflake.to.gcs.sf.query cannot be provided as arguments at the same time.")

        return vars(known_args)

    def get_read_options(self, logger: Logger, args: Dict[str, Any]) -> "tuple[DataFrame, DataFrame]":

        # Arguments
        sf_url: str = args[constants.SNOWFLAKE_TO_GCS_SF_URL]
        sf_user: str = args[constants.SNOWFLAKE_TO_GCS_SF_USER]
        sf_pwd: str = args[constants.SNOWFLAKE_TO_GCS_SF_PASSWORD]
        sf_database: str = args[constants.SNOWFLAKE_TO_GCS_SF_DATABASE]
        sf_schema: str = args[constants.SNOWFLAKE_TO_GCS_SF_SCHEMA]
        sf_warehouse: str = args[constants.SNOWFLAKE_TO_GCS_SF_WAREHOUSE]
        sf_autopushdown: str = args[constants.SNOWFLAKE_TO_GCS_SF_AUTOPUSHDOWN]
        sf_table: str = args[constants.SNOWFLAKE_TO_GCS_SF_TABLE]
        sf_query: str = args[constants.SNOWFLAKE_TO_GCS_SF_QUERY]

        logger.info(
            "Starting Snowflake to Cloud Storage Spark job with parameters:\n"
            f"{pprint.pformat(args)}"
        )

        sf_options = {
                     "sfURL" : sf_url,
                     "sfUser" : sf_user,
                     "sfPassword" : sf_pwd,
                     "sfDatabase" : sf_database,
                     "sfSchema" : sf_schema,
                     "sfWarehouse" : sf_warehouse,
                     "autopushdown" : sf_autopushdown
                    }

        data_options = {
            "dbtable" : sf_table,
            "query" : sf_query
        }

        return sf_options, data_options


    def read_data(self, logger: Logger, spark: SparkSession, sf_opt: Dict[str,Any], data_opt: Dict[str,Any] ) -> DataFrame:

        if not sf_opt or not data_opt:
            sys.exit("There seems to be an issue in fetching read options. Read options cannot be empty \n")

        # Read
        logger.info(
            "Starting process of reading data from source \n"
        )

        input_data: DataFrame = spark.read.format(constants.FORMAT_SNOWFLAKE) \
            .options(**sf_opt)

        if data_opt["dbtable"]:
            input_data = input_data.option("dbtable",data_opt["dbtable"])
        else:
            input_data = input_data.option("query",data_opt["query"])

        input_data = input_data.load()
        count = input_data.count()

        if not count :
            sys.exit("The input dataframe is empty. The table is either empty or there is no data for the selected filters")
        else:
            logger.info(
            "Data from source has been read successfully \n"
            )

        return input_data


    def write_data(self, logger: Logger, args: Dict[str, Any], input_data: DataFrame) -> None:

        output_format: str = args[constants.SNOWFLAKE_TO_GCS_OUTPUT_FORMAT]
        output_location: str = args[constants.SNOWFLAKE_TO_GCS_OUTPUT_LOCATION]
        output_mode: str = args[constants.SNOWFLAKE_TO_GCS_OUTPUT_MODE]
        partition_col: str = args[constants.SNOWFLAKE_TO_GCS_PARTITION_COLUMN]

        # Write
        logger.info(
            "Starting process of writing data to Cloud Storage \n"
        )

        if partition_col:
            writer: DataFrameWriter = input_data.write.mode(output_mode) \
                .partitionBy(partition_col)
        else:
            writer: DataFrameWriter = input_data.write.mode(output_mode)

        persist_dataframe_to_cloud_storage(writer, args, output_location, output_format, "snowflake.gcs.output.")

        logger.info(
            "Data from source has been loaded to Cloud Storage successfully"
        )


    def run(self, spark: SparkSession, args: Dict[str, Any]) -> None:

        logger: Logger = self.get_logger(spark=spark)

        sf_options, data_options = self.get_read_options(logger, args)

        input_data = self.read_data(logger, spark, sf_options, data_options)

        self.write_data(logger, args, input_data)



from typing import Dict, Sequence, Optional, Any
from logging import Logger
import argparse
import pprint
import os


from pyspark.sql import SparkSession, DataFrame


from dataproc_templates import BaseTemplate
import dataproc_templates.util.template_constants as constants

__all__ = ['KafkaToBigQueryTemplate']

class KafkaToBigQueryTemplate(BaseTemplate):
    """
    Dataproc template implementing loads from Kafka into Bigquery
    """
    @staticmethod
    def parse_args(args: Optional[Sequence[str]] = None) -> Dict[str, Any]:
        parser: argparse.ArgumentParser = argparse.ArgumentParser()

        parser.add_argument(
            f'--{constants.KAFKA_BQ_CHECKPOINT_LOCATION}',
            dest=constants.KAFKA_BQ_CHECKPOINT_LOCATION,
            required=True,
            help='GCS location of the checkpoint folder'
        )
        parser.add_argument(
            f'--{constants.KAFKA_BOOTSTRAP_SERVERS}',
            dest=constants.KAFKA_BOOTSTRAP_SERVERS,
            required=True,
            help='Kafka topic address from where data is coming'
        )
        parser.add_argument(
            f'--{constants.KAFKA_BQ_TOPIC}',
            dest=constants.KAFKA_BQ_TOPIC,
            required=True,
            help='Kafka Topic Name'
        )
        parser.add_argument(
            f'--{constants.KAFKA_BQ_STARTING_OFFSET}',
            dest=constants.KAFKA_BQ_STARTING_OFFSET,
            required=True,
            help='Offset to start reading from. Accepted values: "earliest", "latest","{json string}"}'
        )
        parser.add_argument(
            f'--{constants.KAFKA_BQ_DATASET}',
            dest=constants.KAFKA_BQ_DATASET,
            required=True,
            help='Bigquery Dataset'
        )
        parser.add_argument(
            f'--{constants.KAFKA_BQ_TABLE_NAME}',
            dest=constants.KAFKA_BQ_TABLE_NAME,
            required=True,
            help="Bigquery Table Name"
        )
        parser.add_argument(
            f'--{constants.KAFKA_BQ_OUTPUT_MODE}',
            dest=constants.KAFKA_BQ_OUTPUT_MODE,
            required=True,
            help="Bigquery Table Output Mode (append , complete, update)"
        )
        parser.add_argument(
            f'--{constants.KAFKA_BQ_TEMP_BUCKET_NAME}',
            dest=constants.KAFKA_BQ_TEMP_BUCKET_NAME,
            required=True,
            help="GCS Temp Bucket Name"
        )
        parser.add_argument(
            f'--{constants.KAFKA_BQ_TERMINATION_TIMEOUT}',
            dest=constants.KAFKA_BQ_TERMINATION_TIMEOUT,
            required=True,
            help="Timeout for termination of kafka subscription"
        )

        known_args: argparse.Namespace
        known_args, _ = parser.parse_known_args(args)

        return vars(known_args)

    def run(self, spark: SparkSession, args: Dict[str, Any]) -> None:

        logger: Logger = self.get_logger(spark=spark)

        logger.info(
            "Starting Kafka to Bigquery Pyspark job with parameters:\n"
            f"{pprint.pformat(args)}"
        )

        #arguments
        bootstrap_server_list: str = args[constants.KAFKA_BOOTSTRAP_SERVERS]
        checkpoint_location: str = args[constants.KAFKA_BQ_CHECKPOINT_LOCATION]
        kafka_topics: str= args[constants.KAFKA_BQ_TOPIC]
        big_query_dataset: str = args[constants.KAFKA_BQ_DATASET]
        big_query_table: str = args[constants.KAFKA_BQ_TABLE_NAME]
        bq_temp_bucket: str = args[constants.KAFKA_BQ_TEMP_BUCKET_NAME]
        timeout: int = int(args[constants.KAFKA_BQ_TERMINATION_TIMEOUT])
        offset:str = args[constants.KAFKA_BQ_STARTING_OFFSET]
        output_mode = args[constants.KAFKA_BQ_OUTPUT_MODE]

    
        df = spark.readStream.format(constants.KAFKA_INPUT_FORMAT) \
                  .option('kafka.bootstrap.servers', bootstrap_server_list) \
                  .option('subscribe', kafka_topics) \
                  .option('startingOffsets',offset) \
                  .load()
        
        df = df.selectExpr("CAST(key AS STRING)", "CAST(value AS STRING)")


        # Write
        output = df \
        .writeStream \
        .format(constants.FORMAT_BIGQUERY) \
        .outputMode(output_mode) \
        .option('checkpointLocation',checkpoint_location) \
        .option('table',big_query_dataset+'.'+big_query_table) \
        .option('temporaryGcsBucket', bq_temp_bucket) \
        .start() 
        
        output.awaitTermination(timeout)







# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Optional, Sequence, Union
import argparse

import dataproc_templates.util.template_constants as constants
from dataproc_templates import TemplateName


def get_template_name(args: Optional[Sequence[str]] = None) -> TemplateName:
    """
    Parses the template name option from the program arguments.

    This will print the command help and exit if the --template
    argument is missing.

    Args:
        args (Optional[Sequence[str]]): The program arguments.
            By default, command line arguments will be used.

    Returns:
        str: The value of the --template argument
    """

    parser: argparse.ArgumentParser = argparse.ArgumentParser(
        add_help=False
    )

    parser.add_argument(
        '--template',
        dest='template_name',
        type=str,
        required=False,
        default=None,
        choices=TemplateName.choices(),
        help='The name of the template to run'
    )

    known_args: argparse.Namespace
    known_args, _ = parser.parse_known_args(args=args)

    if known_args.template_name is None:
        parser.print_help()
        parser.exit()

    return TemplateName.from_string(known_args.template_name)


def get_log_level(args: Optional[Sequence[str]] = None) -> str:
    """
    Parses the log level option from the program arguments.

    INFO is the default log level
    This will exit if the log level in an invalid choice.

    Args:
        args (Optional[Sequence[str]]): The program arguments.
            By default, command line arguments will be used.

    Returns:
        str: The value of the --log_level argument
    """

    parser: argparse.ArgumentParser = argparse.ArgumentParser(
        add_help=False
    )

    parser.add_argument(
        '--log_level',
        dest='log_level',
        type=str,
        required=False,
        default="INFO",
        choices=["ALL", "DEBUG", "ERROR", "FATAL", "INFO", "OFF", "TRACE", "WARN"],
        help='Spark Context Log Level'
    )

    known_args: argparse.Namespace
    known_args, _ = parser.parse_known_args(args=args)

    return known_args.log_level


def add_spark_options(parser: argparse.ArgumentParser, template_to_spark_option_map: dict, read_options: bool = True) -> None:
    if not template_to_spark_option_map:
        return

    for option_name, spark_option_name in template_to_spark_option_map.items():
        if read_options:
            help_text = (constants.SPARK_OPTIONS[spark_option_name].get(constants.OPTION_READ_HELP, "")
                         or constants.SPARK_OPTIONS[spark_option_name].get(constants.OPTION_HELP, ""))
        else:
            help_text = (constants.SPARK_OPTIONS[spark_option_name].get(constants.OPTION_WRITE_HELP, "")
                         or constants.SPARK_OPTIONS[spark_option_name].get(constants.OPTION_HELP, ""))
        parser.add_argument(
            f'--{option_name}',
            dest=option_name,
            required=False,
            default=constants.SPARK_OPTIONS[spark_option_name].get(constants.OPTION_DEFAULT, ""),
            help=help_text
        )


"""
 * Copyright 2022 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
"""

import mock
import pyspark

from dataproc_templates.gcs.gcs_to_gcs import GCSToGCSTemplate
import dataproc_templates.util.template_constants as constants


class TestGCSToGCSTemplate:
    """
    Test suite for GCSToGCSTemplate
    """

    def test_parse_args(self):
        gcs_to_gcs_template = GCSToGCSTemplate()
        parsed_args = gcs_to_gcs_template.parse_args(
            ["--gcs.to.gcs.input.location=gs://input",
             "--gcs.to.gcs.input.format=csv",
             "--gcs.to.gcs.temp.view.name=temp",
             "--gcs.to.gcs.sql.query=select * from temp",
             "--gcs.to.gcs.output.format=csv",
             "--gcs.to.gcs.output.mode=overwrite",
             "--gcs.to.gcs.output.partition.column=column",
             "--gcs.to.gcs.output.location=gs://output"])

        assert parsed_args["gcs.to.gcs.input.location"] == "gs://input"
        assert parsed_args["gcs.to.gcs.input.format"] == "csv"
        assert parsed_args["gcs.to.gcs.temp.view.name"] == "temp"
        assert parsed_args["gcs.to.gcs.sql.query"] == "select * from temp"
        assert parsed_args["gcs.to.gcs.output.format"] == "csv"
        assert parsed_args["gcs.to.gcs.output.mode"] == "overwrite"
        assert parsed_args["gcs.to.gcs.output.partition.column"] == "column"
        assert parsed_args["gcs.to.gcs.output.location"] == "gs://output"

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_parquet(self, mock_spark_session):
        """Tests GCSToGCSTemplate runs with parquet format"""

        gcs_to_gcs_template = GCSToGCSTemplate()
        mock_parsed_args = gcs_to_gcs_template.parse_args(
            ["--gcs.to.gcs.input.location=gs://input",
             "--gcs.to.gcs.input.format=parquet",
             "--gcs.to.gcs.temp.view.name=temp",
             "--gcs.to.gcs.sql.query=select * from temp",
             "--gcs.to.gcs.output.format=parquet",
             "--gcs.to.gcs.output.mode=overwrite",
             "--gcs.to.gcs.output.partition.column=column",
             "--gcs.to.gcs.output.location=gs://output"])

        mock_spark_session.read.parquet.return_value = mock_spark_session.dataframe.DataFrame
        gcs_to_gcs_template.run(mock_spark_session, mock_parsed_args)

        mock_spark_session.read.parquet.assert_called_once_with("gs://input")
        mock_spark_session.read.parquet().createOrReplaceTempView.assert_called_once_with("temp")
        mock_spark_session.sql.assert_called_once_with("select * from temp")
        mock_spark_session.sql().write \
            .mode.assert_called_once_with(constants.OUTPUT_MODE_OVERWRITE)
        mock_spark_session.sql().write \
            .mode() \
            .partitionBy.assert_called_once_with("column")
        mock_spark_session.sql().write \
            .mode() \
            .partitionBy() \
            .parquet.assert_called_once_with("gs://output")

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_csv(self, mock_spark_session):
        """Tests GCSToGCSTemplate runs with parquet format"""

        gcs_to_gcs_template = GCSToGCSTemplate()
        mock_parsed_args = gcs_to_gcs_template.parse_args(
            ["--gcs.to.gcs.input.location=gs://input",
             "--gcs.to.gcs.input.format=csv",
             "--gcs.to.gcs.temp.view.name=temp",
             "--gcs.to.gcs.sql.query=select * from temp",
             "--gcs.to.gcs.output.format=csv",
             "--gcs.to.gcs.output.mode=overwrite",
             "--gcs.to.gcs.output.partition.column=column",
             "--gcs.to.gcs.output.location=gs://output"])

        mock_spark_session.read.csv.return_value = mock_spark_session.dataframe.DataFrame
        gcs_to_gcs_template.run(mock_spark_session, mock_parsed_args)

        mock_spark_session.read.format.assert_called_once_with(constants.FORMAT_CSV)
        mock_spark_session.read.format() \
            .options.assert_called_once_with(**{constants.CSV_HEADER: 'true',
                                                constants.CSV_INFER_SCHEMA: 'true'})
        mock_spark_session.read.format() \
            .options() \
            .load.assert_called_once_with("gs://input")
        mock_spark_session.read.format() \
            .options() \
            .load() \
            .createOrReplaceTempView.assert_called_once_with("temp")

        mock_spark_session.sql.assert_called_once_with("select * from temp")
        mock_spark_session.sql().write \
            .mode.assert_called_once_with(constants.OUTPUT_MODE_OVERWRITE)
        mock_spark_session.sql().write \
            .mode() \
            .partitionBy.assert_called_once_with("column")
        mock_spark_session.sql().write \
            .mode() \
            .partitionBy() \
            .options.assert_called_once_with(**{constants.CSV_HEADER: 'true'})

        mock_spark_session.sql().write \
            .mode() \
            .partitionBy() \
            .options() \
            .csv.assert_called_once_with("gs://output")

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_avro(self, mock_spark_session):
        """Tests GCSToGCSTemplate runs with avro format"""

        gcs_to_gcs_template = GCSToGCSTemplate()
        mock_parsed_args = gcs_to_gcs_template.parse_args(
            ["--gcs.to.gcs.input.location=gs://input",
             "--gcs.to.gcs.input.format=avro",
             "--gcs.to.gcs.temp.view.name=temp",
             "--gcs.to.gcs.sql.query=select * from temp",
             "--gcs.to.gcs.output.format=avro",
             "--gcs.to.gcs.output.mode=overwrite",
             "--gcs.to.gcs.output.partition.column=column",
             "--gcs.to.gcs.output.location=gs://output"])

        mock_spark_session.read.csv.return_value = mock_spark_session.dataframe.DataFrame
        gcs_to_gcs_template.run(mock_spark_session, mock_parsed_args)

        mock_spark_session.read.format.assert_called_once_with(constants.FORMAT_AVRO)

        mock_spark_session.read.format() \
            .load.assert_called_once_with("gs://input")
        mock_spark_session.read.format() \
            .load() \
            .createOrReplaceTempView.assert_called_once_with("temp")

        mock_spark_session.sql.assert_called_once_with("select * from temp")
        mock_spark_session.sql().write \
            .mode.assert_called_once_with(constants.OUTPUT_MODE_OVERWRITE)
        mock_spark_session.sql().write \
            .mode() \
            .partitionBy.assert_called_once_with("column")
        mock_spark_session.sql().write \
            .mode() \
            .partitionBy() \
            .format.assert_called_once_with(constants.FORMAT_AVRO)

        mock_spark_session.sql().write \
            .mode() \
            .partitionBy() \
            .format() \
            .save.assert_called_once_with("gs://output")

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_json(self, mock_spark_session):
        """Tests GCSToGCSTemplate runs with json format"""

        gcs_to_gcs_template = GCSToGCSTemplate()
        mock_parsed_args = gcs_to_gcs_template.parse_args(
            ["--gcs.to.gcs.input.location=gs://input",
             "--gcs.to.gcs.input.format=json",
             "--gcs.to.gcs.temp.view.name=temp",
             "--gcs.to.gcs.sql.query=select * from temp",
             "--gcs.to.gcs.output.format=json",
             "--gcs.to.gcs.output.mode=overwrite",
             "--gcs.to.gcs.output.partition.column=column",
             "--gcs.to.gcs.output.location=gs://output"])

        mock_spark_session.read.json.return_value = mock_spark_session.dataframe.DataFrame
        gcs_to_gcs_template.run(mock_spark_session, mock_parsed_args)

        mock_spark_session.read.json.assert_called_once_with("gs://input")
        mock_spark_session.read.json().createOrReplaceTempView.assert_called_once_with("temp")
        mock_spark_session.sql.assert_called_once_with("select * from temp")
        mock_spark_session.sql().write \
            .mode.assert_called_once_with(constants.OUTPUT_MODE_OVERWRITE)
        mock_spark_session.sql().write \
            .mode() \
            .partitionBy.assert_called_once_with("column")
        mock_spark_session.sql().write \
            .mode() \
            .partitionBy() \
            .json.assert_called_once_with("gs://output")

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_delta(self, mock_spark_session):
        """Tests GCSToGCSTemplate runs with delta format"""

        gcs_to_gcs_template = GCSToGCSTemplate()
        mock_parsed_args = gcs_to_gcs_template.parse_args(
            ["--gcs.to.gcs.input.location=gs://input",
             "--gcs.to.gcs.input.format=delta",
             "--gcs.to.gcs.temp.view.name=temp",
             "--gcs.to.gcs.sql.query=select * from temp",
             "--gcs.to.gcs.output.format=avro",
             "--gcs.to.gcs.output.mode=overwrite",
             "--gcs.to.gcs.output.partition.column=column",
             "--gcs.to.gcs.output.location=gs://output"])

        mock_spark_session.read.csv.return_value = mock_spark_session.dataframe.DataFrame
        gcs_to_gcs_template.run(mock_spark_session, mock_parsed_args)

        mock_spark_session.read.format.assert_called_once_with(constants.FORMAT_DELTA)

        mock_spark_session.read.format() \
            .load.assert_called_once_with("gs://input")
        mock_spark_session.read.format() \
            .load() \
            .createOrReplaceTempView.assert_called_once_with("temp")

        mock_spark_session.sql.assert_called_once_with("select * from temp")
        mock_spark_session.sql().write \
            .mode.assert_called_once_with(constants.OUTPUT_MODE_OVERWRITE)
        mock_spark_session.sql().write \
            .mode() \
            .partitionBy.assert_called_once_with("column")
        mock_spark_session.sql().write \
            .mode() \
            .partitionBy() \
            .format.assert_called_once_with(constants.FORMAT_AVRO)

        mock_spark_session.sql().write \
            .mode() \
            .partitionBy() \
            .format() \
            .save.assert_called_once_with("gs://output")



"""
 * Copyright 2022 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
"""

import mock
import pyspark

from dataproc_templates.gcs.gcs_to_bigquery import GCSToBigQueryTemplate
import dataproc_templates.util.template_constants as constants


class TestGCSToBigQueryTemplate:
    """
    Test suite for GCSToBigQueryTemplate
    """

    def test_parse_args(self):
        """Tests GCSToBigQueryTemplate.parse_args()"""

        gcs_to_bigquery_template = GCSToBigQueryTemplate()
        parsed_args = gcs_to_bigquery_template.parse_args(
            ["--gcs.bigquery.input.format=parquet",
             "--gcs.bigquery.output.mode=append",
             "--gcs.bigquery.input.location=gs://test",
             "--gcs.bigquery.output.dataset=dataset",
             "--gcs.bigquery.output.table=table",
             "--gcs.bigquery.temp.bucket.name=bucket"])

        assert parsed_args["gcs.bigquery.input.format"] == "parquet"
        assert parsed_args["gcs.bigquery.output.mode"] == "append"
        assert parsed_args["gcs.bigquery.input.location"] == "gs://test"
        assert parsed_args["gcs.bigquery.output.dataset"] == "dataset"
        assert parsed_args["gcs.bigquery.output.table"] == "table"
        assert parsed_args["gcs.bigquery.temp.bucket.name"] == "bucket"

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_parquet(self, mock_spark_session):
        """Tests GCSToBigqueryTemplate runs with parquet format"""

        gcs_to_bigquery_template = GCSToBigQueryTemplate()
        mock_parsed_args = gcs_to_bigquery_template.parse_args(
            ["--gcs.bigquery.input.format=parquet",
             "--gcs.bigquery.output.mode=append",
             "--gcs.bigquery.input.location=gs://test",
             "--gcs.bigquery.output.dataset=dataset",
             "--gcs.bigquery.output.table=table",
             "--gcs.bigquery.temp.bucket.name=bucket"])
        mock_spark_session.read.parquet.return_value = mock_spark_session.dataframe.DataFrame
        gcs_to_bigquery_template.run(mock_spark_session, mock_parsed_args)

        mock_spark_session.read.parquet.assert_called_once_with("gs://test")
        mock_spark_session.dataframe.DataFrame.write.format.assert_called_once_with(
            constants.FORMAT_BIGQUERY)
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option.assert_called_once_with(constants.TABLE, "dataset.table")
        mock_spark_session.dataframe.DataFrame.write.format().option(
        ).option.assert_called_once_with(constants.GCS_BQ_TEMP_BUCKET, "bucket")
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option().mode.assert_called_once_with(constants.OUTPUT_MODE_APPEND)
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option().mode().save.assert_called_once()


    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_json(self, mock_spark_session):
        """Tests GCSToBigqueryTemplate runs with json format"""

        gcs_to_bigquery_template = GCSToBigQueryTemplate()
        mock_parsed_args = gcs_to_bigquery_template.parse_args(
            ["--gcs.bigquery.input.format=json",
             "--gcs.bigquery.output.mode=errorifexists",
             "--gcs.bigquery.input.location=gs://test",
             "--gcs.bigquery.output.dataset=dataset",
             "--gcs.bigquery.output.table=table",
             "--gcs.bigquery.temp.bucket.name=bucket"])
        mock_spark_session.read.json.return_value = mock_spark_session.dataframe.DataFrame
        gcs_to_bigquery_template.run(mock_spark_session, mock_parsed_args)

        mock_spark_session.read.json.assert_called_once_with("gs://test")
        mock_spark_session.dataframe.DataFrame.write.format.assert_called_once_with(
            constants.FORMAT_BIGQUERY)
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option.assert_called_once_with(constants.TABLE, "dataset.table")
        mock_spark_session.dataframe.DataFrame.write.format().option(
        ).option.assert_called_once_with(constants.GCS_BQ_TEMP_BUCKET, "bucket")
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option().mode.assert_called_once_with(constants.OUTPUT_MODE_ERRORIFEXISTS)
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option().mode().save.assert_called_once()

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_avro(self, mock_spark_session):
        """Tests GCSToBigqueryTemplate runs with avro"""

        gcs_to_bigquery_template = GCSToBigQueryTemplate()
        mock_parsed_args = gcs_to_bigquery_template.parse_args(
            ["--gcs.bigquery.input.format=avro",
             "--gcs.bigquery.output.mode=overwrite",
             "--gcs.bigquery.input.location=gs://test",
             "--gcs.bigquery.output.dataset=dataset",
             "--gcs.bigquery.output.table=table",
             "--gcs.bigquery.temp.bucket.name=bucket"])
        mock_spark_session.read.format().load.return_value = mock_spark_session.dataframe.DataFrame
        gcs_to_bigquery_template.run(mock_spark_session, mock_parsed_args)

        mock_spark_session.read.format.assert_called_with(
            constants.FORMAT_AVRO_EXTD)
        mock_spark_session.read.format().load.assert_called_once_with("gs://test")
        mock_spark_session.dataframe.DataFrame.write.format.assert_called_once_with(
            constants.FORMAT_BIGQUERY)
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option.assert_called_once_with(constants.TABLE, "dataset.table")
        mock_spark_session.dataframe.DataFrame.write.format().option(
        ).option.assert_called_once_with(constants.GCS_BQ_TEMP_BUCKET, "bucket")
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option().mode.assert_called_once_with(constants.OUTPUT_MODE_OVERWRITE)
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option().mode().save.assert_called_once()

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_delta(self, mock_spark_session):
        """Tests GCSToBigqueryTemplate runs with delta format"""

        gcs_to_bigquery_template = GCSToBigQueryTemplate()
        mock_parsed_args = gcs_to_bigquery_template.parse_args(
            ["--gcs.bigquery.input.format=delta",
             "--gcs.bigquery.output.mode=overwrite",
             "--gcs.bigquery.input.location=gs://test",
             "--gcs.bigquery.output.dataset=dataset",
             "--gcs.bigquery.output.table=table",
             "--gcs.bigquery.temp.bucket.name=bucket"])
        mock_spark_session.read.format().load.return_value = mock_spark_session.dataframe.DataFrame
        gcs_to_bigquery_template.run(mock_spark_session, mock_parsed_args)

        mock_spark_session.read.format.assert_called_with(
            constants.FORMAT_DELTA)
        mock_spark_session.read.format().load.assert_called_once_with("gs://test")
        mock_spark_session.dataframe.DataFrame.write.format.assert_called_once_with(
            constants.FORMAT_BIGQUERY)
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option.assert_called_once_with(constants.TABLE, "dataset.table")
        mock_spark_session.dataframe.DataFrame.write.format().option(
        ).option.assert_called_once_with(constants.GCS_BQ_TEMP_BUCKET, "bucket")
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option().mode.assert_called_once_with(constants.OUTPUT_MODE_OVERWRITE)
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option().mode().save.assert_called_once()

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_csv1(self, mock_spark_session):
        """Tests GCSToBigqueryTemplate runs with csv format"""

        gcs_to_bigquery_template = GCSToBigQueryTemplate()
        mock_parsed_args = gcs_to_bigquery_template.parse_args(
            ["--gcs.bigquery.input.format=csv",
             "--gcs.bigquery.output.mode=ignore",
             "--gcs.bigquery.input.location=gs://test",
             "--gcs.bigquery.output.dataset=dataset",
             "--gcs.bigquery.output.table=table",
             "--gcs.bigquery.temp.bucket.name=bucket"])
        mock_spark_session.read.format().options().load.return_value = mock_spark_session.dataframe.DataFrame
        gcs_to_bigquery_template.run(mock_spark_session, mock_parsed_args)

        mock_spark_session.read.format.assert_called_with(
            constants.FORMAT_CSV)
        mock_spark_session.read.format().options.assert_called_with(**{
            constants.CSV_HEADER: 'true',
            constants.CSV_INFER_SCHEMA: 'true',
        })
        mock_spark_session.read.format().options(
        ).load.assert_called_once_with("gs://test")

        mock_spark_session.dataframe.DataFrame.write.format.assert_called_once_with(
            constants.FORMAT_BIGQUERY)
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option.assert_called_once_with(constants.TABLE, "dataset.table")
        mock_spark_session.dataframe.DataFrame.write.format().option(
        ).option.assert_called_once_with(constants.GCS_BQ_TEMP_BUCKET, "bucket")
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option().mode.assert_called_once_with(constants.OUTPUT_MODE_IGNORE)
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option().mode().save.assert_called_once()

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_csv2(self, mock_spark_session):
        """Tests GCSToBigqueryTemplate runs with csv format and some optional csv options"""

        gcs_to_bigquery_template = GCSToBigQueryTemplate()
        mock_parsed_args = gcs_to_bigquery_template.parse_args(
            ["--gcs.bigquery.input.format=csv",
             "--gcs.bigquery.output.mode=ignore",
             "--gcs.bigquery.input.location=gs://test",
             "--gcs.bigquery.input.inferschema=false",
             "--gcs.bigquery.input.sep=|",
             "--gcs.bigquery.input.comment=#",
             "--gcs.bigquery.input.timestampntzformat=yyyy-MM-dd'T'HH:mm:ss",
             "--gcs.bigquery.output.dataset=dataset",
             "--gcs.bigquery.output.table=table",
             "--gcs.bigquery.temp.bucket.name=bucket"])
        mock_spark_session.read.format().options(
        ).load.return_value = mock_spark_session.dataframe.DataFrame
        gcs_to_bigquery_template.run(mock_spark_session, mock_parsed_args)

        mock_spark_session.read.format.assert_called_with(
            constants.FORMAT_CSV)
        mock_spark_session.read.format().options.assert_called_with(**{
            constants.CSV_HEADER: 'true',
            constants.CSV_INFER_SCHEMA: 'false',
            constants.CSV_SEP: "|",
            constants.CSV_COMMENT: "#",
            constants.CSV_TIMESTAMPNTZFORMAT: "yyyy-MM-dd'T'HH:mm:ss",
        })
        mock_spark_session.read.format().options(
        ).load.assert_called_once_with("gs://test")
        mock_spark_session.dataframe.DataFrame.write.format.assert_called_once_with(
            constants.FORMAT_BIGQUERY)
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option.assert_called_once_with(constants.TABLE, "dataset.table")
        mock_spark_session.dataframe.DataFrame.write.format().option(
        ).option.assert_called_once_with(constants.GCS_BQ_TEMP_BUCKET, "bucket")
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option().mode.assert_called_once_with(constants.OUTPUT_MODE_IGNORE)
        mock_spark_session.dataframe.DataFrame.write.format(
        ).option().option().mode().save.assert_called_once()




"""
 * Copyright 2022 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
"""

import mock
import pyspark
from dataproc_templates.cassandra.cassandra_to_gcs import CassandraToGCSTemplate
import dataproc_templates.util.template_constants as constants


class TestCassandraToGCSTemplate:
    """
    Test suite for CassandraToGCSTemplate
    """

    def test_parse_args1(self):
        """Tests CassandraToGCSTemplate.parse_args()"""

        cassandra_to_gcs_template = CassandraToGCSTemplate()
        parsed_args = cassandra_to_gcs_template.parse_args(
            ["--cassandratogcs.input.table=tablename",
             "--cassandratogcs.input.host=192.168.2.2",
             "--cassandratogcs.output.format=csv",
             "--cassandratogcs.output.path=dataset.table",
             "--cassandratogcs.output.savemode=append",
             "--cassandratogcs.input.catalog.name=casscon",
             "--cassandratogcs.input.keyspace=tk1"
             ])

        assert parsed_args["cassandratogcs.input.table"] == "tablename"
        assert parsed_args["cassandratogcs.input.host"] == "192.168.2.2"
        assert parsed_args["cassandratogcs.output.format"] == "csv"
        assert parsed_args["cassandratogcs.output.path"] == "dataset.table"
        assert parsed_args["cassandratogcs.output.savemode"] == "append"
        assert parsed_args["cassandratogcs.input.catalog.name"] == "casscon"
        assert parsed_args["cassandratogcs.input.keyspace"] == "tk1"


# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Dict, Sequence, Optional, Any
from logging import Logger
import argparse
import pprint
import sys
from pyspark.sql import SparkSession, DataFrame, DataFrameWriter

from dataproc_templates import BaseTemplate
from dataproc_templates.util.argument_parsing import add_spark_options
from dataproc_templates.util.dataframe_writer_wrappers import persist_dataframe_to_cloud_storage
import dataproc_templates.util.template_constants as constants


__all__ = ['JDBCToGCSTemplate']


class JDBCToGCSTemplate(BaseTemplate):
    """
    Dataproc template implementing loads from JDBC into Cloud Storage
    """

    @staticmethod
    def parse_args(args: Optional[Sequence[str]] = None) -> Dict[str, Any]:
        parser: argparse.ArgumentParser = argparse.ArgumentParser()

        parser.add_argument(
            f'--{constants.JDBCTOGCS_INPUT_URL}',
            dest=constants.JDBCTOGCS_INPUT_URL,
            required=True,
            help='JDBC input URL'
        )
        parser.add_argument(
            f'--{constants.JDBCTOGCS_INPUT_DRIVER}',
            dest=constants.JDBCTOGCS_INPUT_DRIVER,
            required=True,
            help='JDBC input driver name'
        )
        parser.add_argument(
            f'--{constants.JDBCTOGCS_INPUT_TABLE}',
            dest=constants.JDBCTOGCS_INPUT_TABLE,
            required=False,
            help='JDBC input table name'
        )
        parser.add_argument(
            f'--{constants.JDBCTOGCS_INPUT_SQL_QUERY}',
            dest=constants.JDBCTOGCS_INPUT_SQL_QUERY,
            required=False,
            help='JDBC input SQL query'
        )
        parser.add_argument(
            f'--{constants.JDBCTOGCS_INPUT_PARTITIONCOLUMN}',
            dest=constants.JDBCTOGCS_INPUT_PARTITIONCOLUMN,
            required=False,
            default="",
            help='JDBC input table partition column name'
        )
        parser.add_argument(
            f'--{constants.JDBCTOGCS_INPUT_LOWERBOUND}',
            dest=constants.JDBCTOGCS_INPUT_LOWERBOUND,
            required=False,
            default="",
            help='JDBC input table partition column lower bound which is used to decide the partition stride'
        )
        parser.add_argument(
            f'--{constants.JDBCTOGCS_INPUT_UPPERBOUND}',
            dest=constants.JDBCTOGCS_INPUT_UPPERBOUND,
            required=False,
            default="",
            help='JDBC input table partition column upper bound which is used to decide the partition stride'
        )
        parser.add_argument(
            f'--{constants.JDBCTOGCS_NUMPARTITIONS}',
            dest=constants.JDBCTOGCS_NUMPARTITIONS,
            required=False,
            default="10",
            help='The maximum number of partitions that can be used for parallelism in table reading and writing. Default set to 10'
        )
        parser.add_argument(
            f'--{constants.JDBCTOGCS_INPUT_FETCHSIZE}',
            dest=constants.JDBCTOGCS_INPUT_FETCHSIZE,
            required=False,
            default=0,
            type=int,
            help='Determines how many rows to fetch per round trip'
        )
        parser.add_argument(
            f'--{constants.JDBCTOGCS_SESSIONINITSTATEMENT}',
            dest=constants.JDBCTOGCS_SESSIONINITSTATEMENT,
            required=False,
            default="",
            help='Custom SQL statement to execute in each reader database session'
        )
        parser.add_argument(
            f'--{constants.JDBCTOGCS_OUTPUT_LOCATION}',
            dest=constants.JDBCTOGCS_OUTPUT_LOCATION,
            required=True,
            help='Cloud Storage location for output files'
        )
        parser.add_argument(
            f'--{constants.JDBCTOGCS_OUTPUT_FORMAT}',
            dest=constants.JDBCTOGCS_OUTPUT_FORMAT,
            required=True,
            help='Output file format (one of: avro,parquet,csv,json)',
            choices=[
                constants.FORMAT_AVRO,
                constants.FORMAT_PRQT,
                constants.FORMAT_CSV,
                constants.FORMAT_JSON
            ]
        )
        parser.add_argument(
            f'--{constants.JDBCTOGCS_OUTPUT_MODE}',
            dest=constants.JDBCTOGCS_OUTPUT_MODE,
            required=False,
            default=constants.OUTPUT_MODE_APPEND,
            help=(
                'Output write mode '
                '(one of: append,overwrite,ignore,errorifexists) '
                '(Defaults to append)'
            ),
            choices=[
                constants.OUTPUT_MODE_OVERWRITE,
                constants.OUTPUT_MODE_APPEND,
                constants.OUTPUT_MODE_IGNORE,
                constants.OUTPUT_MODE_ERRORIFEXISTS
            ]
        )
        parser.add_argument(
            f'--{constants.JDBCTOGCS_OUTPUT_PARTITIONCOLUMN}',
            dest=constants.JDBCTOGCS_OUTPUT_PARTITIONCOLUMN,
            required=False,
            default="",
            help='Cloud Storage partition column name'
        )
        parser.add_argument(
            f'--{constants.JDBCTOGCS_TEMP_VIEW_NAME}',
            dest=constants.JDBCTOGCS_TEMP_VIEW_NAME,
            required=False,
            default="",
            help='Temp view name for creating a spark sql view on source data. This name has to match with the table name that will be used in the SQL query'
        )
        parser.add_argument(
            f'--{constants.JDBCTOGCS_TEMP_SQL_QUERY}',
            dest=constants.JDBCTOGCS_TEMP_SQL_QUERY,
            required=False,
            default="",
            help='SQL query for data transformation. This must use the temp view name as the table to query from.'
        )
        add_spark_options(parser, constants.get_csv_output_spark_options("jdbc.gcs.output."))

        known_args: argparse.Namespace
        known_args, _ = parser.parse_known_args(args)

        if getattr(known_args, constants.JDBCTOGCS_INPUT_TABLE) and getattr(known_args, constants.JDBCTOGCS_INPUT_SQL_QUERY):
            sys.exit('ArgumentParser Error: Arguments cannot have both input table and sql query, use either one.')

        if getattr(known_args, constants.JDBCTOGCS_TEMP_SQL_QUERY) and not getattr(known_args, constants.JDBCTOGCS_TEMP_VIEW_NAME):
            sys.exit('ArgumentParser Error: Temp view name cannot be null if you want to do data transformations with query')

        return vars(known_args)

    def run(self, spark: SparkSession, args: Dict[str, Any]) -> None:

        logger: Logger = self.get_logger(spark=spark)

        # Arguments
        input_jdbc_url: str = args[constants.JDBCTOGCS_INPUT_URL]
        input_jdbc_driver: str = args[constants.JDBCTOGCS_INPUT_DRIVER]
        input_jdbc_table: str = args[constants.JDBCTOGCS_INPUT_TABLE]
        input_jdbc_sql_query: str = args[constants.JDBCTOGCS_INPUT_SQL_QUERY]
        input_jdbc_partitioncolumn: str = args[constants.JDBCTOGCS_INPUT_PARTITIONCOLUMN]
        input_jdbc_lowerbound: str = args[constants.JDBCTOGCS_INPUT_LOWERBOUND]
        input_jdbc_upperbound: str = args[constants.JDBCTOGCS_INPUT_UPPERBOUND]
        jdbc_numpartitions: str = args[constants.JDBCTOGCS_NUMPARTITIONS]
        input_jdbc_fetchsize: int = args[constants.JDBCTOGCS_INPUT_FETCHSIZE]
        input_jdbc_sessioninitstatement: str = args[constants.JDBCTOGCS_SESSIONINITSTATEMENT]
        output_location: str = args[constants.JDBCTOGCS_OUTPUT_LOCATION]
        output_format: str = args[constants.JDBCTOGCS_OUTPUT_FORMAT]
        output_mode: str = args[constants.JDBCTOGCS_OUTPUT_MODE]
        output_partitioncolumn: str = args[constants.JDBCTOGCS_OUTPUT_PARTITIONCOLUMN]
        temp_view: str = args[constants.JDBCTOGCS_TEMP_VIEW_NAME]
        temp_sql_query:str = args[constants.JDBCTOGCS_TEMP_SQL_QUERY]

        logger.info(
            "Starting JDBC to Cloud Storage Spark job with parameters:\n"
            f"{pprint.pformat(args)}"
        )

        # Read
        input_data: DataFrame

        read_properties = {constants.JDBC_URL: input_jdbc_url,
                           constants.JDBC_DRIVER: input_jdbc_driver}

        if input_jdbc_table:
            read_properties.update({constants.JDBC_TABLE: input_jdbc_table})
        elif input_jdbc_sql_query:
            read_properties.update({constants.JDBC_QUERY: input_jdbc_sql_query})
        else:
            logger.error("Arguments must have either input table or input SQL query")
            exit(1)

        read_properties.update({constants.JDBC_NUMPARTITIONS: jdbc_numpartitions,
                                constants.JDBC_FETCHSIZE: input_jdbc_fetchsize})

        if input_jdbc_sessioninitstatement:
            read_properties[constants.JDBC_SESSIONINITSTATEMENT] = input_jdbc_sessioninitstatement

        partition_parameters = str(input_jdbc_partitioncolumn) + str(input_jdbc_lowerbound) + str(input_jdbc_upperbound)
        if ((partition_parameters != "") & ((input_jdbc_partitioncolumn == "") | (input_jdbc_lowerbound == "") | (input_jdbc_upperbound == ""))):
            logger.error("Set all the sql partitioning parameters together-jdbctogcs.input.partitioncolumn,jdbctogcs.input.lowerbound,jdbctogcs.input.upperbound. Refer to README.md for more instructions.")
            exit(1)

        if partition_parameters:
            read_properties.update({constants.JDBC_PARTITIONCOLUMN: input_jdbc_partitioncolumn,
                                    constants.JDBC_LOWERBOUND: input_jdbc_lowerbound,
                                    constants.JDBC_UPPERBOUND: input_jdbc_upperbound})

        input_data = spark.read \
            .format(constants.FORMAT_JDBC) \
            .options(**read_properties) \
            .load()

        if temp_sql_query:
            # Create temp view on source data
            input_data.createGlobalTempView(temp_view)
            # Execute SQL
            output_data = spark.sql(temp_sql_query)
        else:
            output_data = input_data

        # Write
        if (output_partitioncolumn != ""):
            writer: DataFrameWriter = output_data.write.mode(output_mode).partitionBy(output_partitioncolumn)
        else:
            writer: DataFrameWriter = output_data.write.mode(output_mode)

        persist_dataframe_to_cloud_storage(writer, args, output_location, output_format, "jdbc.gcs.output.")


from typing import Dict, Sequence, Optional, Any
from logging import Logger
import argparse
import pprint




from pyspark.sql import SparkSession, DataFrame


from dataproc_templates import BaseTemplate
import dataproc_templates.util.template_constants as constants

__all__ = ['KafkaToGCSTemplate']

class KafkaToGCSTemplate(BaseTemplate):
    """
    Dataproc template implementing loads from Kafka into GCS
    """
    @staticmethod
    def parse_args(args: Optional[Sequence[str]] = None) -> Dict[str, Any]:
        parser: argparse.ArgumentParser = argparse.ArgumentParser()

        parser.add_argument(
            f'--{constants.KAFKA_GCS_CHECKPOINT_LOCATION}',
            dest=constants.KAFKA_GCS_CHECKPOINT_LOCATION,
            required=True,
            help='Checkpoint location for Kafka to GCS Template'
        )
        parser.add_argument(
            f'--{constants.KAFKA_GCS_OUTPUT_LOCATION}',
            dest=constants.KAFKA_GCS_OUTPUT_LOCATION,
            required=True,
            help='GCS location of the destination folder'
        )
        parser.add_argument(
            f'--{constants.KAFKA_GCS_BOOTSTRAP_SERVERS}',
            dest=constants.KAFKA_GCS_BOOTSTRAP_SERVERS,
            required=True,
            help='Kafka topic address from where data is coming'
        )
        parser.add_argument(
            f'--{constants.KAFKA_TOPIC}',
            dest=constants.KAFKA_TOPIC,
            required=True,
            help='Kafka Topic Name'
        )
        parser.add_argument(
            f'--{constants.KAFKA_STARTING_OFFSET}',
            dest=constants.KAFKA_STARTING_OFFSET,
            required=True,
            help='Starting offset value (earliest, latest, json_string)'
        )
        parser.add_argument(
            f'--{constants.KAFKA_GCS_OUTPUT_FORMAT}',
            dest=constants.KAFKA_GCS_OUTPUT_FORMAT,
            required=True,
            help='Ouput format of the data (json , csv, avro, parquet)'
        )
        parser.add_argument(
            f'--{constants.KAFKA_GCS_OUPUT_MODE}',
            dest=constants.KAFKA_GCS_OUPUT_MODE,
            required=True,
            help="Ouput type of GCS append|overwrite"
        )
        parser.add_argument(
            f'--{constants.KAFKA_GCS_TERMINATION_TIMEOUT}',
            dest=constants.KAFKA_GCS_TERMINATION_TIMEOUT,
            required=True,
            help="Timeout for termination of kafka subscription"
        )



        known_args: argparse.Namespace
        known_args, _ = parser.parse_known_args(args)

        return vars(known_args)

    def run(self, spark: SparkSession, args: Dict[str, Any]) -> None:

        logger: Logger = self.get_logger(spark=spark)

        #arguments
        bootstrap_server_list: str = args[constants.KAFKA_GCS_BOOTSTRAP_SERVERS]
        gcs_output_location: str = args[constants.KAFKA_GCS_OUTPUT_LOCATION]
        kafka_topics: str= args[constants.KAFKA_TOPIC]
        output_format: str= args[constants.KAFKA_GCS_OUTPUT_FORMAT]
        output_mode:str = args[constants.KAFKA_GCS_OUPUT_MODE]
        timeout: int = int(args[constants.KAFKA_GCS_TERMINATION_TIMEOUT])
        offset:str = args[constants.KAFKA_STARTING_OFFSET]
        checkpoint_loc: str = args[constants.KAFKA_GCS_CHECKPOINT_LOCATION]

        logger.info(
            "Starting Kafka to GCS Pyspark job with parameters:\n"
            f"{pprint.pformat(args)}"
        )


        df = spark.readStream.format(constants.KAFKA_INPUT_FORMAT) \
                  .option('kafka.bootstrap.servers', bootstrap_server_list) \
                  .option('subscribe', kafka_topics) \
                  .option('startingOffsets',offset) \
                  .load()
        
        df = df.selectExpr("CAST(key AS STRING)", "CAST(value AS STRING)")
        # Write
        
        df \
        .writeStream \
        .format(output_format) \
        .outputMode(output_mode) \
        .option('checkpointLocation',checkpoint_loc) \
        .option('path',gcs_output_location) \
        .start() \
        .awaitTermination(timeout)



# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Dict, Sequence, Optional, Any
from logging import Logger
import argparse
import pprint
import sys

from pyspark.sql import SparkSession, DataFrameWriter

from dataproc_templates import BaseTemplate
import dataproc_templates.util.template_constants as constants

__all__ = ['CassandraToBQTemplate']


class CassandraToBQTemplate(BaseTemplate):
    """
    Dataproc template implementing exports from Cassandra to BQ
    """

    @staticmethod
    def parse_args(args: Optional[Sequence[str]] = None) -> Dict[str, Any]:
        parser: argparse.ArgumentParser = argparse.ArgumentParser()

        parser.add_argument(
            f'--{constants.CASSANDRA_TO_BQ_INPUT_TABLE}',
            dest=constants.CASSANDRA_TO_BQ_INPUT_TABLE,
            required=False,
            help='Cassandra to BQ Input table name'
        )
        parser.add_argument(
            f'--{constants.CASSANDRA_TO_BQ_INPUT_HOST}',
            dest=constants.CASSANDRA_TO_BQ_INPUT_HOST,
            required=True,
            help='Input hostname for Cassandra cluster'
        )
        parser.add_argument(
            f'--{constants.CASSANDRA_TO_BQ_BIGQUERY_LOCATION}',
            dest=constants.CASSANDRA_TO_BQ_BIGQUERY_LOCATION,
            required=True,
            help='Target table in BQ Format: <dataset>.<table-name>'
        )
        parser.add_argument(
            f'--{constants.CASSANDRA_TO_BQ_WRITE_MODE}',
            dest=constants.CASSANDRA_TO_BQ_WRITE_MODE,
            required=False,
            default=constants.OUTPUT_MODE_APPEND,
            help=(
                'Output write mode '
                '(one of: append,overwrite,ignore,errorifexists) '
                '(Defaults to append)'
            ),
            choices=[
                constants.OUTPUT_MODE_OVERWRITE,
                constants.OUTPUT_MODE_APPEND,
                constants.OUTPUT_MODE_IGNORE,
                constants.OUTPUT_MODE_ERRORIFEXISTS
            ]
        )
        parser.add_argument(
            f'--{constants.CASSANDRA_TO_BQ_TEMP_LOCATION}',
            dest=constants.CASSANDRA_TO_BQ_TEMP_LOCATION,
            required=True,
            help='Cloud Storage location for staging, Format: <bucket-name>'
        )
        parser.add_argument(
            f'--{constants.CASSANDRA_TO_BQ_QUERY}',
            dest=constants.CASSANDRA_TO_BQ_QUERY,
            required=False,
            help='Optional query for selective exports'
        )
        parser.add_argument(
            f'--{constants.CASSANDRA_TO_BQ_CATALOG}',
            dest=constants.CASSANDRA_TO_BQ_CATALOG,
            required=False,
            default="casscon",
            help='To provide a name for connection between Cassandra and BQ'
        )
        parser.add_argument(
            f'--{constants.CASSANDRA_TO_BQ_INPUT_KEYSPACE}',
            dest=constants.CASSANDRA_TO_BQ_INPUT_KEYSPACE,
            required=False,
            help='Keyspace Name of Cassandra Table'
        )

        known_args: argparse.Namespace
        known_args, _ = parser.parse_known_args(args)

        if (not getattr(known_args, constants.CASSANDRA_TO_BQ_QUERY)
            and (not getattr(known_args, constants.CASSANDRA_TO_BQ_INPUT_KEYSPACE)
            or not getattr(known_args, constants.CASSANDRA_TO_BQ_INPUT_TABLE))):

            sys.exit("ArgumentParser Error: Either of cassandratobq.input.keyspace and cassandratobq.input.table "
                        + "OR cassandratobq.input.query needs to be provided as argument to read data from Cassandra")

        elif (getattr(known_args, constants.CASSANDRA_TO_BQ_QUERY)
            and (getattr(known_args, constants.CASSANDRA_TO_BQ_INPUT_KEYSPACE)
            or getattr(known_args, constants.CASSANDRA_TO_BQ_INPUT_TABLE))):

            sys.exit("ArgumentParser Error: Both cassandratobq.input.keyspace and cassandratobq.input.table "
                        + "AND cassandratobq.input.query cannot be provided as arguments at the same time.")

        return vars(known_args)

    def run(self, spark: SparkSession, args: Dict[str, Any]) -> None:

        logger: Logger = self.get_logger(spark=spark)

        # Arguments
        input_table: str = args[constants.CASSANDRA_TO_BQ_INPUT_TABLE]
        input_host: str = args[constants.CASSANDRA_TO_BQ_INPUT_HOST]
        output_mode: str = args[constants.CASSANDRA_TO_BQ_WRITE_MODE]
        output_location: str = args[constants.CASSANDRA_TO_BQ_BIGQUERY_LOCATION]
        catalog: str = args[constants.CASSANDRA_TO_BQ_CATALOG]
        query: str = args[constants.CASSANDRA_TO_BQ_QUERY]
        tempLocation: str = args[constants.CASSANDRA_TO_BQ_TEMP_LOCATION]
        input_keyspace: str = args[constants.CASSANDRA_TO_BQ_INPUT_KEYSPACE]

        logger.info(
            "Starting Cassandra to Bigquery spark job with parameters:\n"
            f"{pprint.pformat(args)}"
        )
        # Set configuration to connect to Cassandra by overwriting the spark session
        spark = (
            SparkSession
                .builder
                .appName("CassandraToBQ")
                .config(constants.SQL_EXTENSION, constants.CASSANDRA_EXTENSION)
                .config(f"spark.sql.catalog.{catalog}", constants.CASSANDRA_CATALOG)
                .config(f"spark.sql.catalog.{catalog}.spark.cassandra.connection.host",input_host)
                .getOrCreate())
        # Read
        if(not query):
            input_data = spark.read.table(f"{catalog}.{input_keyspace}.{input_table}")
        else:
            input_data= spark.sql(query)
        # Write
        input_data.write.format(constants.FORMAT_BIGQUERY)\
            .mode(output_mode).option(constants.TABLE, output_location)\
            .option(constants.TEMP_GCS_BUCKET, tempLocation).save()


"""
 * Copyright 2022 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
"""

import mock
import pyspark

from dataproc_templates.jdbc.jdbc_to_jdbc import JDBCToJDBCTemplate
import dataproc_templates.util.template_constants as constants


class TestJDBCToJDBCTemplate:
    """
    Test suite for JDBCToJDBCTemplate
    """

    def test_parse_args(self):
        """Tests JDBCToJDBCTemplate.parse_args()"""

        jdbc_to_jdbc_template = JDBCToJDBCTemplate()
        parsed_args = jdbc_to_jdbc_template.parse_args(
            ["--jdbctojdbc.input.url=url",
             "--jdbctojdbc.input.driver=driver",
             "--jdbctojdbc.input.table=table1",
             "--jdbctojdbc.input.partitioncolumn=column",
             "--jdbctojdbc.input.lowerbound=1",
             "--jdbctojdbc.input.upperbound=2",
             "--jdbctojdbc.numpartitions=5",
             "--jdbctojdbc.input.fetchsize=100",
             "--jdbctojdbc.input.sessioninitstatement=EXEC some_setup_sql('data1')",
             "--jdbctojdbc.output.url=url",
             "--jdbctojdbc.output.driver=driver",
             "--jdbctojdbc.output.table=table2",
             "--jdbctojdbc.output.create_table.option=dummy",
             "--jdbctojdbc.output.mode=append",
             "--jdbctojdbc.output.batch.size=1000"
             ])

        assert parsed_args["jdbctojdbc.input.url"] == "url"
        assert parsed_args["jdbctojdbc.input.driver"] == "driver"
        assert parsed_args["jdbctojdbc.input.table"] == "table1"
        assert parsed_args["jdbctojdbc.input.partitioncolumn"] == "column"
        assert parsed_args["jdbctojdbc.input.lowerbound"] == "1"
        assert parsed_args["jdbctojdbc.input.upperbound"] == "2"
        assert parsed_args["jdbctojdbc.numpartitions"] == "5"
        assert parsed_args["jdbctojdbc.input.fetchsize"] == 100
        assert parsed_args["jdbctojdbc.input.sessioninitstatement"] == "EXEC some_setup_sql('data1')"
        assert parsed_args["jdbctojdbc.output.url"] == "url"
        assert parsed_args["jdbctojdbc.output.driver"] == "driver"
        assert parsed_args["jdbctojdbc.output.table"] == "table2"
        assert parsed_args["jdbctojdbc.output.create_table.option"] == "dummy"
        assert parsed_args["jdbctojdbc.output.mode"] == "append"
        assert parsed_args["jdbctojdbc.output.batch.size"] == "1000"


    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_pass_args1(self, mock_spark_session):
        """Tests JDBCToJDBCTemplate pass args"""

        jdbc_to_jdbc_template = JDBCToJDBCTemplate()

        mock_parsed_args = jdbc_to_jdbc_template.parse_args(
            ["--jdbctojdbc.input.url=url",
             "--jdbctojdbc.input.driver=driver",
             "--jdbctojdbc.input.table=table1",
             "--jdbctojdbc.input.partitioncolumn=column",
             "--jdbctojdbc.input.lowerbound=1",
             "--jdbctojdbc.input.upperbound=2",
             "--jdbctojdbc.numpartitions=5",
             "--jdbctojdbc.input.sessioninitstatement=EXEC some_setup_sql('data2')",
             "--jdbctojdbc.output.url=url",
             "--jdbctojdbc.output.driver=driver",
             "--jdbctojdbc.output.table=table2",
             "--jdbctojdbc.output.create_table.option=dummy",
             "--jdbctojdbc.output.mode=append",
             "--jdbctojdbc.output.batch.size=1000"
             ])
        mock_spark_session.read.format().options().load.return_value = mock_spark_session.dataframe.DataFrame
        jdbc_to_jdbc_template.run(mock_spark_session, mock_parsed_args)
        mock_spark_session.read.format.assert_called_with(constants.FORMAT_JDBC)
        _, kwargs = mock_spark_session.read.format().options.call_args
        assert (constants.JDBC_URL, "url") in kwargs.items()
        assert (constants.JDBC_DRIVER, "driver") in kwargs.items()
        assert (constants.JDBC_TABLE, "table1") in kwargs.items()
        assert (constants.JDBC_PARTITIONCOLUMN, "column") in kwargs.items()
        assert (constants.JDBC_LOWERBOUND, "1") in kwargs.items()
        assert (constants.JDBC_UPPERBOUND, "2") in kwargs.items()
        assert (constants.JDBC_NUMPARTITIONS, "5") in kwargs.items()
        assert (constants.JDBC_FETCHSIZE, 0) in kwargs.items()
        assert (constants.JDBC_SESSIONINITSTATEMENT, "EXEC some_setup_sql('data2')") in kwargs.items()
        mock_spark_session.read.format().options().load()
        mock_spark_session.dataframe.DataFrame.write.format.assert_called_once_with(constants.FORMAT_JDBC)
        mock_spark_session.dataframe.DataFrame.write.format().option.assert_called_once_with(constants.JDBC_URL, "url")
        mock_spark_session.dataframe.DataFrame.write.format().option().option.assert_called_once_with(constants.JDBC_DRIVER, "driver")
        mock_spark_session.dataframe.DataFrame.write.format().option().option().option.assert_called_once_with(constants.JDBC_TABLE, "table2")
        mock_spark_session.dataframe.DataFrame.write.format().option().option().option().option.assert_called_once_with(constants.JDBC_CREATE_TABLE_OPTIONS, "dummy")
        mock_spark_session.dataframe.DataFrame.write.format().option().option().option().option().option.assert_called_once_with(constants.JDBC_BATCH_SIZE, "1000")
        mock_spark_session.dataframe.DataFrame.write.format().option().option().option().option().option().option.assert_called_once_with(constants.JDBC_NUMPARTITIONS, "5")
        mock_spark_session.dataframe.DataFrame.write.format().option().option().option().option().option().option().mode.assert_called_once_with(constants.OUTPUT_MODE_APPEND)
        mock_spark_session.dataframe.DataFrame.write.format().option().option().option().option().option().option().mode().save.assert_called_once()


    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_pass_args2(self, mock_spark_session):
        """Tests JDBCToJDBCTemplate pass args"""

        jdbc_to_jdbc_template = JDBCToJDBCTemplate()

        mock_parsed_args = jdbc_to_jdbc_template.parse_args(
            ["--jdbctojdbc.input.url=url",
             "--jdbctojdbc.input.driver=driver",
             "--jdbctojdbc.input.table=table1",
             "--jdbctojdbc.input.partitioncolumn=column",
             "--jdbctojdbc.input.lowerbound=1",
             "--jdbctojdbc.input.upperbound=2",
             "--jdbctojdbc.numpartitions=5",
             "--jdbctojdbc.input.fetchsize=100",
             "--jdbctojdbc.output.url=url",
             "--jdbctojdbc.output.driver=driver",
             "--jdbctojdbc.output.table=table2"
             ])
        mock_spark_session.read.format().options().load.return_value = mock_spark_session.dataframe.DataFrame
        jdbc_to_jdbc_template.run(mock_spark_session, mock_parsed_args)
        mock_spark_session.read.format.assert_called_with(constants.FORMAT_JDBC)
        _, kwargs = mock_spark_session.read.format().options.call_args
        assert (constants.JDBC_URL, "url") in kwargs.items()
        assert (constants.JDBC_DRIVER, "driver") in kwargs.items()
        assert (constants.JDBC_TABLE, "table1") in kwargs.items()
        assert (constants.JDBC_PARTITIONCOLUMN, "column") in kwargs.items()
        assert (constants.JDBC_LOWERBOUND, "1") in kwargs.items()
        assert (constants.JDBC_UPPERBOUND, "2") in kwargs.items()
        assert (constants.JDBC_NUMPARTITIONS, "5") in kwargs.items()
        assert (constants.JDBC_FETCHSIZE, 100) in kwargs.items()
        assert constants.JDBC_SESSIONINITSTATEMENT not in kwargs
        mock_spark_session.read.format().options().load()
        mock_spark_session.dataframe.DataFrame.write.format.assert_called_once_with(constants.FORMAT_JDBC)
        mock_spark_session.dataframe.DataFrame.write.format().option.assert_called_once_with(constants.JDBC_URL, "url")
        mock_spark_session.dataframe.DataFrame.write.format().option().option.assert_called_once_with(constants.JDBC_DRIVER, "driver")
        mock_spark_session.dataframe.DataFrame.write.format().option().option().option.assert_called_once_with(constants.JDBC_TABLE, "table2")
        mock_spark_session.dataframe.DataFrame.write.format().option().option().option().option.assert_called_once_with(constants.JDBC_CREATE_TABLE_OPTIONS, "")
        mock_spark_session.dataframe.DataFrame.write.format().option().option().option().option().option.assert_called_once_with(constants.JDBC_BATCH_SIZE, "1000")
        mock_spark_session.dataframe.DataFrame.write.format().option().option().option().option().option().option.assert_called_once_with(constants.JDBC_NUMPARTITIONS, "5")
        mock_spark_session.dataframe.DataFrame.write.format().option().option().option().option().option().option().mode.assert_called_once_with(constants.OUTPUT_MODE_APPEND)
        mock_spark_session.dataframe.DataFrame.write.format().option().option().option().option().option().option().mode().save.assert_called_once()

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_pass_args3(self, mock_spark_session):
        """Tests JDBCToJDBCTemplate pass args"""

        jdbc_to_jdbc_template = JDBCToJDBCTemplate()

        mock_parsed_args = jdbc_to_jdbc_template.parse_args(
            ["--jdbctojdbc.input.url=url",
             "--jdbctojdbc.input.driver=driver",
             "--jdbctojdbc.input.table=table1",
             "--jdbctojdbc.input.fetchsize=20",
             "--jdbctojdbc.output.url=url",
             "--jdbctojdbc.output.driver=driver",
             "--jdbctojdbc.output.table=table2"
             ])
        mock_spark_session.read.format().options().load.return_value = mock_spark_session.dataframe.DataFrame
        jdbc_to_jdbc_template.run(mock_spark_session, mock_parsed_args)
        mock_spark_session.read.format.assert_called_with(constants.FORMAT_JDBC)
        _, kwargs = mock_spark_session.read.format().options.call_args
        assert (constants.JDBC_URL, "url") in kwargs.items()
        assert (constants.JDBC_DRIVER, "driver") in kwargs.items()
        assert (constants.JDBC_TABLE, "table1") in kwargs.items()
        assert (constants.JDBC_NUMPARTITIONS, "10") in kwargs.items()
        assert (constants.JDBC_FETCHSIZE, 20) in kwargs.items()
        assert constants.JDBC_SESSIONINITSTATEMENT not in kwargs
        mock_spark_session.read.format().options().load()
        mock_spark_session.dataframe.DataFrame.write.format.assert_called_once_with(constants.FORMAT_JDBC)
        mock_spark_session.dataframe.DataFrame.write.format().option.assert_called_once_with(constants.JDBC_URL, "url")
        mock_spark_session.dataframe.DataFrame.write.format().option().option.assert_called_once_with(constants.JDBC_DRIVER, "driver")
        mock_spark_session.dataframe.DataFrame.write.format().option().option().option.assert_called_once_with(constants.JDBC_TABLE, "table2")
        mock_spark_session.dataframe.DataFrame.write.format().option().option().option().option.assert_called_once_with(constants.JDBC_CREATE_TABLE_OPTIONS, "")
        mock_spark_session.dataframe.DataFrame.write.format().option().option().option().option().option.assert_called_once_with(constants.JDBC_BATCH_SIZE, "1000")
        mock_spark_session.dataframe.DataFrame.write.format().option().option().option().option().option().option.assert_called_once_with(constants.JDBC_NUMPARTITIONS, "10")
        mock_spark_session.dataframe.DataFrame.write.format().option().option().option().option().option().option().mode.assert_called_once_with(constants.OUTPUT_MODE_APPEND)
        mock_spark_session.dataframe.DataFrame.write.format().option().option().option().option().option().option().mode().save.assert_called_once()




"""
 * Copyright 2022 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
"""

import mock
import pyspark

from dataproc_templates.jdbc.jdbc_to_gcs import JDBCToGCSTemplate
import dataproc_templates.util.template_constants as constants


class TestJDBCToGCSTemplate:
    """
    Test suite for JDBCToGCSTemplate
    """

    def test_parse_args1(self):
        """Tests JDBCToGCSTemplate.parse_args()"""

        jdbc_to_gcs_template = JDBCToGCSTemplate()
        parsed_args = jdbc_to_gcs_template.parse_args(
            ["--jdbctogcs.input.url=url",
             "--jdbctogcs.input.driver=driver",
             "--jdbctogcs.input.table=table1",
             "--jdbctogcs.input.partitioncolumn=column",
             "--jdbctogcs.input.lowerbound=1",
             "--jdbctogcs.input.upperbound=2",
             "--jdbctogcs.numpartitions=5",
             "--jdbctogcs.input.fetchsize=100",
             "--jdbctogcs.input.sessioninitstatement=EXEC some_setup_sql('data1')",
             "--jdbctogcs.output.location=gs://test",
             "--jdbctogcs.output.format=csv",
             "--jdbctogcs.output.mode=append",
             "--jdbctogcs.output.partitioncolumn=column"
             ])

        assert parsed_args["jdbctogcs.input.url"] == "url"
        assert parsed_args["jdbctogcs.input.driver"] == "driver"
        assert parsed_args["jdbctogcs.input.table"] == "table1"
        assert parsed_args["jdbctogcs.input.partitioncolumn"] == "column"
        assert parsed_args["jdbctogcs.input.lowerbound"] == "1"
        assert parsed_args["jdbctogcs.input.upperbound"] == "2"
        assert parsed_args["jdbctogcs.numpartitions"] == "5"
        assert parsed_args["jdbctogcs.input.fetchsize"] == 100
        assert parsed_args["jdbctogcs.input.sessioninitstatement"] == "EXEC some_setup_sql('data1')"
        assert parsed_args["jdbctogcs.output.location"] == "gs://test"
        assert parsed_args["jdbctogcs.output.format"] == "csv"
        assert parsed_args["jdbctogcs.output.mode"] == "append"
        assert parsed_args["jdbctogcs.output.partitioncolumn"] == "column"

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_pass_args2(self, mock_spark_session):
        """Tests JDBCToGCSTemplate write parquet"""

        jdbc_to_gcs_template = JDBCToGCSTemplate()

        mock_parsed_args = jdbc_to_gcs_template.parse_args(
            ["--jdbctogcs.input.url=url",
             "--jdbctogcs.input.driver=driver",
             "--jdbctogcs.input.table=table1",
             "--jdbctogcs.input.partitioncolumn=column",
             "--jdbctogcs.input.lowerbound=1",
             "--jdbctogcs.input.upperbound=2",
             "--jdbctogcs.numpartitions=5",
             "--jdbctogcs.input.sessioninitstatement=EXEC some_setup_sql('data2')",
             "--jdbctogcs.output.location=gs://test",
             "--jdbctogcs.output.format=parquet",
             "--jdbctogcs.output.mode=overwrite"
             ])
        mock_spark_session.read.format().options().load.return_value = mock_spark_session.dataframe.DataFrame
        jdbc_to_gcs_template.run(mock_spark_session, mock_parsed_args)
        mock_spark_session.read.format.assert_called_with(
            constants.FORMAT_JDBC)
        mock_spark_session.read.format().options.assert_called_with(**{
            constants.JDBC_URL: "url",
            constants.JDBC_DRIVER: "driver",
            constants.JDBC_TABLE: "table1",
            constants.JDBC_PARTITIONCOLUMN: "column",
            constants.JDBC_LOWERBOUND: "1",
            constants.JDBC_UPPERBOUND: "2",
            constants.JDBC_NUMPARTITIONS: "5",
            constants.JDBC_FETCHSIZE: 0,
            constants.JDBC_SESSIONINITSTATEMENT: "EXEC some_setup_sql('data2')"
        })
        mock_spark_session.read.format().options().load()
        mock_spark_session.dataframe.DataFrame.write.mode.assert_called_once_with(constants.OUTPUT_MODE_OVERWRITE)
        mock_spark_session.dataframe.DataFrame.write.mode().parquet.assert_called_once_with("gs://test")

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_pass_args3(self, mock_spark_session):
        """Tests JDBCToGCSTemplate write avro"""

        jdbc_to_gcs_template = JDBCToGCSTemplate()

        mock_parsed_args = jdbc_to_gcs_template.parse_args(
            ["--jdbctogcs.input.url=url",
             "--jdbctogcs.input.driver=driver",
             "--jdbctogcs.input.table=table1",
             "--jdbctogcs.input.partitioncolumn=column",
             "--jdbctogcs.input.lowerbound=1",
             "--jdbctogcs.input.upperbound=2",
             "--jdbctogcs.numpartitions=5",
             "--jdbctogcs.input.fetchsize=50",
             "--jdbctogcs.output.location=gs://test",
             "--jdbctogcs.output.format=avro",
             "--jdbctogcs.output.mode=append"
             ])
        mock_spark_session.read.format().options().load.return_value = mock_spark_session.dataframe.DataFrame
        jdbc_to_gcs_template.run(mock_spark_session, mock_parsed_args)
        mock_spark_session.read.format.assert_called_with(
            constants.FORMAT_JDBC)
        mock_spark_session.read.format().options.assert_called_with(**{
            constants.JDBC_URL: "url",
            constants.JDBC_DRIVER: "driver",
            constants.JDBC_TABLE: "table1",
            constants.JDBC_PARTITIONCOLUMN: "column",
            constants.JDBC_LOWERBOUND: "1",
            constants.JDBC_UPPERBOUND: "2",
            constants.JDBC_NUMPARTITIONS: "5",
            constants.JDBC_FETCHSIZE: 50
        })
        mock_spark_session.read.format().options().load()
        mock_spark_session.dataframe.DataFrame.write.mode.assert_called_once_with(constants.OUTPUT_MODE_APPEND)
        mock_spark_session.dataframe.DataFrame.write.mode().format.assert_called_once_with(constants.FORMAT_AVRO)
        mock_spark_session.dataframe.DataFrame.write.mode().format().save.assert_called_once_with("gs://test")

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_pass_args4(self, mock_spark_session):
        """Tests JDBCToGCSTemplate write csv"""

        jdbc_to_gcs_template = JDBCToGCSTemplate()

        mock_parsed_args = jdbc_to_gcs_template.parse_args(
            ["--jdbctogcs.input.url=url",
             "--jdbctogcs.input.driver=driver",
             "--jdbctogcs.input.table=table1",
             "--jdbctogcs.input.partitioncolumn=column",
             "--jdbctogcs.input.lowerbound=1",
             "--jdbctogcs.input.upperbound=2",
             "--jdbctogcs.numpartitions=5",
             "--jdbctogcs.output.location=gs://test",
             "--jdbctogcs.output.format=csv",
             "--jdbctogcs.output.mode=ignore"
             ])
        mock_spark_session.read.format().options().load.return_value = mock_spark_session.dataframe.DataFrame
        jdbc_to_gcs_template.run(mock_spark_session, mock_parsed_args)
        mock_spark_session.read.format.assert_called_with(
            constants.FORMAT_JDBC)
        mock_spark_session.read.format().options.assert_called_with(**{
            constants.JDBC_URL: "url",
            constants.JDBC_DRIVER: "driver",
            constants.JDBC_TABLE: "table1",
            constants.JDBC_PARTITIONCOLUMN: "column",
            constants.JDBC_LOWERBOUND: "1",
            constants.JDBC_UPPERBOUND: "2",
            constants.JDBC_NUMPARTITIONS: "5",
            constants.JDBC_FETCHSIZE: 0
        })
        mock_spark_session.read.format().options().load()
        mock_spark_session.dataframe.DataFrame.write.mode.assert_called_once_with(constants.OUTPUT_MODE_IGNORE)
        mock_spark_session.dataframe.DataFrame.write.mode().options.assert_called_once_with(**{constants.CSV_HEADER: 'true'})
        mock_spark_session.dataframe.DataFrame.write.mode().options().csv.assert_called_once_with("gs://test")

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_pass_args5(self, mock_spark_session):
        """Tests JDBCToGCSTemplate write json"""

        jdbc_to_gcs_template = JDBCToGCSTemplate()

        mock_parsed_args = jdbc_to_gcs_template.parse_args(
            ["--jdbctogcs.input.url=url",
             "--jdbctogcs.input.driver=driver",
             "--jdbctogcs.input.table=table1",
             "--jdbctogcs.input.partitioncolumn=column",
             "--jdbctogcs.input.lowerbound=1",
             "--jdbctogcs.input.upperbound=2",
             "--jdbctogcs.numpartitions=5",
             "--jdbctogcs.output.location=gs://test",
             "--jdbctogcs.output.format=json",
             "--jdbctogcs.output.mode=ignore"
             ])
        mock_spark_session.read.format().options().load.return_value = mock_spark_session.dataframe.DataFrame
        jdbc_to_gcs_template.run(mock_spark_session, mock_parsed_args)
        mock_spark_session.read.format.assert_called_with(constants.FORMAT_JDBC)
        mock_spark_session.read.format().options.assert_called_with(**{
            constants.JDBC_URL: "url",
            constants.JDBC_DRIVER: "driver",
            constants.JDBC_TABLE: "table1",
            constants.JDBC_PARTITIONCOLUMN: "column",
            constants.JDBC_LOWERBOUND: "1",
            constants.JDBC_UPPERBOUND: "2",
            constants.JDBC_NUMPARTITIONS: "5",
            constants.JDBC_FETCHSIZE: 0
        })
        mock_spark_session.read.format().options().load()
        mock_spark_session.dataframe.DataFrame.write.mode.assert_called_once_with(constants.OUTPUT_MODE_IGNORE)
        #mock_spark_session.dataframe.DataFrame.write.mode().json.assert_called_once_with("gs://test")

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_pass_args6(self, mock_spark_session):
        """Tests JDBCToGCSTemplate pass args"""

        jdbc_to_gcs_template = JDBCToGCSTemplate()

        mock_parsed_args = jdbc_to_gcs_template.parse_args(
            ["--jdbctogcs.input.url=url",
             "--jdbctogcs.input.driver=driver",
             "--jdbctogcs.input.table=table1",
             "--jdbctogcs.output.location=gs://test",
             "--jdbctogcs.output.format=csv",
             "--jdbctogcs.output.mode=append"
             ])
        mock_spark_session.read.format().options().load.return_value = mock_spark_session.dataframe.DataFrame
        jdbc_to_gcs_template.run(mock_spark_session, mock_parsed_args)
        mock_spark_session.read.format.assert_called_with(constants.FORMAT_JDBC)
        mock_spark_session.read.format().options.assert_called_with(**{
            constants.JDBC_URL: "url",
            constants.JDBC_DRIVER: "driver",
            constants.JDBC_TABLE: "table1",
            constants.JDBC_NUMPARTITIONS: "10",
            constants.JDBC_FETCHSIZE: 0
        })
        mock_spark_session.read.format().options().load()
        mock_spark_session.dataframe.DataFrame.write.mode.assert_called_once_with(constants.OUTPUT_MODE_APPEND)
        mock_spark_session.dataframe.DataFrame.write.mode().options.assert_called_once_with(**{constants.CSV_HEADER: 'true'})
        mock_spark_session.dataframe.DataFrame.write.mode().options().csv.assert_called_once_with("gs://test")

    @mock.patch.object(pyspark.sql, 'SparkSession')
    def test_run_pass_args7(self, mock_spark_session):
        """Tests JDBCToGCSTemplate pass args"""

        jdbc_to_gcs_template = JDBCToGCSTemplate()

        mock_parsed_args = jdbc_to_gcs_template.parse_args(
            ["--jdbctogcs.input.url=url",
             "--jdbctogcs.input.driver=driver",
             "--jdbctogcs.input.table=table1",
             "--jdbctogcs.output.location=gs://test",
             "--jdbctogcs.output.format=csv",
             "--jdbctogcs.output.mode=append",
             "--jdbctogcs.output.partitioncolumn=column"
             ])
        mock_spark_session.read.format().options().load.return_value = mock_spark_session.dataframe.DataFrame
        jdbc_to_gcs_template.run(mock_spark_session, mock_parsed_args)
        mock_spark_session.read.format.assert_called_with(
            constants.FORMAT_JDBC)
        mock_spark_session.read.format().options.assert_called_with(**{
            constants.JDBC_URL: "url",
            constants.JDBC_DRIVER: "driver",
            constants.JDBC_TABLE: "table1",
            constants.JDBC_NUMPARTITIONS: "10",
            constants.JDBC_FETCHSIZE: 0
        })
        mock_spark_session.read.format().options().load()
        mock_spark_session.dataframe.DataFrame.write.mode.assert_called_once_with(constants.OUTPUT_MODE_APPEND)
        mock_spark_session.dataframe.DataFrame.write.mode().partitionBy.assert_called_once_with("column")
        mock_spark_session.dataframe.DataFrame.write.mode().partitionBy().options.assert_called_once_with(**{constants.CSV_HEADER: 'true'})
        mock_spark_session.dataframe.DataFrame.write.mode().partitionBy().options().csv.assert_called_once_with("gs://test")
